@@ -121,15 +121,16 @@ const char* const ColorScheme::translatedColorNames[TABLE_COLORS] =
 };
 
 ColorScheme::ColorScheme()
+    : _opacity(1.0)
+    , _table(nullptr)
+    , _randomTable(nullptr)
 {
-    _table = nullptr;
-    _randomTable = nullptr;
-    _opacity = 1.0;
 }
+
 ColorScheme::ColorScheme(const ColorScheme& other)
-      : _opacity(other._opacity)
-       ,_table(nullptr)
-       ,_randomTable(nullptr)
+    : _opacity(other._opacity)
+    , _table(nullptr)
+    , _randomTable(nullptr)
 {
     setName(other.name());
     setDescription(other.description());
@@ -515,16 +516,16 @@ ColorScheme* KDE3ColorSchemeReader::read()
         if ( line.startsWith(QLatin1String("color")) )
         {
             if (!readColorLine(line,scheme))
-                qDebug() << "Failed to read KDE 3 color scheme line" << line;
+                qWarning() << "Failed to read KDE 3 color scheme line" << line;
         }
         else if ( line.startsWith(QLatin1String("title")) )
         {
             if (!readTitleLine(line,scheme))
-                qDebug() << "Failed to read KDE 3 color scheme title line" << line;
+                qWarning() << "Failed to read KDE 3 color scheme title line" << line;
         }
         else
         {
-            qDebug() << "KDE 3 color scheme contains an unsupported feature, '" <<
+            qWarning() << "KDE 3 color scheme contains an unsupported feature, '" <<
                 line << "'";
         }
     }
@@ -613,10 +614,10 @@ void ColorSchemeManager::loadAllColorSchemes()
     }
 
     if ( failed > 0 ){
-        qDebug() << "failed to load " << failed << " color schemes.";
+        qWarning() << "failed to load " << failed << " color schemes.";
     }
 
-    qDebug() << "load all color schemes";
+    qInfo() << "load all color schemes";
     //qDebug() << "nativeColorSchemes" << nativeColorSchemes ;
     //qDebug() << "kde3ColorSchemes" << kde3ColorSchemes;
     _haveLoadedAll = true;
@@ -643,7 +644,7 @@ bool ColorSchemeManager::loadKDE3ColorScheme(const QString& filePath)
 
     if (scheme->name().isEmpty())
     {
-        qDebug() << "color scheme name is not valid.";
+        qWarning() << "color scheme name is not valid.";
         delete scheme;
         return false;
     }
@@ -654,7 +655,7 @@ bool ColorSchemeManager::loadKDE3ColorScheme(const QString& filePath)
         _colorSchemes.insert(scheme->name(),scheme);
     else
     {
-        qDebug() << "color scheme with name" << scheme->name() << "has already been" <<
+        qWarning() << "color scheme with name" << scheme->name() << "has already been" <<
             "found, ignoring.";
         delete scheme;
     }
@@ -704,7 +705,7 @@ bool ColorSchemeManager::loadColorScheme(const QString& filePath)
 
     if (scheme->name().isEmpty())
     {
-        qDebug() << "Color scheme in" << filePath << "does not have a valid name and was not loaded.";
+        qWarning() << "Color scheme in" << filePath << "does not have a valid name and was not loaded.";
         delete scheme;
         return false;
     }
@@ -715,7 +716,7 @@ bool ColorSchemeManager::loadColorScheme(const QString& filePath)
     }
     else
     {
-        qDebug() << "color scheme with name" << schemeName << "has already been" <<
+        qWarning() << "color scheme with name" << schemeName << "has already been" <<
             "found, ignoring.";
 
         delete scheme;
@@ -780,10 +781,43 @@ bool ColorSchemeManager::deleteColorScheme(const QString& name)
     }
     else
     {
-        qDebug() << "Failed to remove color scheme -" << path;
+        qWarning() << "Failed to remove color scheme -" << path;
         return false;
     }
 }
+
+/*******************************************************************************
+ 1. @函数:    realodColorScheme
+ 2. @作者:    ut000125 sunchengxi
+ 3. @日期:    2020-12-01
+ 4. @说明:    重新加载主题
+*******************************************************************************/
+void ColorSchemeManager::realodColorScheme(const QString &origName)
+{
+    if (!origName.endsWith(QLatin1String(".colorscheme")) || !QFile::exists(origName)) {
+        return;
+    }
+
+    QFileInfo info(origName);
+    const QString &schemeName = info.baseName();
+    ColorScheme *scheme = new ColorScheme();
+    scheme->setName(schemeName);
+    scheme->read(origName);
+
+    if (scheme->name().isEmpty()) {
+        qWarning() << "Color scheme in" << origName << "does not have a valid name and was not loaded.";
+        delete scheme;
+        return;
+    }
+
+    if (_colorSchemes.contains(schemeName)) {
+        qDebug() << "(_colorSchemes.contains(schemeName))";
+        const ColorScheme *needDeletScheme = _colorSchemes[schemeName];
+        delete needDeletScheme;
+        _colorSchemes[schemeName] = scheme;
+    }
+}
+
 QString ColorSchemeManager::findColorSchemePath(const QString& name) const
 {
 //    QString path = KStandardDirs::locate("data","konsole/"+name+".colorscheme");
