@@ -160,6 +160,10 @@ public:
      * Returns the total number of lines, including those stored in the history.
      */
     int lineCount() const;
+    /**
+     * Returns the total number of columns.
+     */
+    int columnCount() const;
 
     /**
      * Sets the history store used by this emulation.  When new lines
@@ -298,7 +302,7 @@ public slots:
      * @param buffer A string of characters received from the terminal program.
      * @param len The length of @p buffer
      */
-    void receiveData(const char *buffer, int len);
+    void receiveData(const char *buffer, int len, bool isCommandExec);
 
 signals:
 
@@ -309,7 +313,7 @@ signals:
      * @param data The buffer of data ready to be sent
      * @param len The length of @p data in bytes
      */
-    void sendData(const char *data, int len);
+    void sendData(const char *data, int len, const QTextCodec *codec);
 
     /**
      * Requests that sending of input to the emulation
@@ -361,6 +365,8 @@ signals:
      */
     void programUsesMouseChanged(bool usesMouse);
 
+    void enableAlternateScrolling(bool enable);
+
     void programBracketedPasteModeChanged(bool bracketedPasteMode);
 
     /**
@@ -449,6 +455,12 @@ signals:
      */
     void flowControlKeyPressed(bool suspendKeyPressed);
 
+    /**
+     * Emitted when the active screen is switched, to indicate whether the primary
+     * screen is in use.
+     */
+    void primaryScreenInUse(bool use);
+
     /**
      * Emitted when the cursor shape or its blinking state is changed via
      * DECSCUSR sequences.
@@ -503,6 +515,14 @@ protected:
     /*const */KeyboardTranslator *_keyTranslator; // the keyboard layout
     /********************* Modify by ut000610 daizhengwen End ************************/
 
+    /**
+       @brief 检测当前iconv使用的GB18030编码是否为2005标准，2005标准强制使用上层补丁版本
+            通过检测2005和2022编码转的差异，以附录D中的编码为例验证
+            2005标准 0xFE51 --> \u20087
+            2022标准 0xFE51 --> \uE816
+       @return iconv使用GB18030编码是否为2005标准，默认返回true
+     */
+    bool detectIconvUse2005Standard();
 protected slots:
     /**
      * Schedules an update of attached views.
@@ -511,6 +531,9 @@ protected slots:
      */
     void bufferedUpdate();
 
+    // used to emit the primaryScreenInUse(bool) signal
+    void checkScreenInUse();
+
 private slots:
 
     // triggered by timer, causes the emulation to send an updated screen image to each
@@ -519,15 +542,23 @@ private slots:
 
     void usesMouseChanged(bool usesMouse);
 
+    void setAlternateScrolling(bool enable);
+
     void bracketedPasteModeChanged(bool bracketedPasteMode);
 
 private:
     bool _usesMouse;
+    bool _alternateScrolling;
     bool _bracketedPasteMode;
     QTimer _bulkTimer1;
     QTimer _bulkTimer2;
 
     int _sessionId;
+
+    /******** Add by ut001000 renfeixiang 2020-07-16:增加保存上一次的屏幕行列数，用于比较终端屏宽是否发生变化 Begin***************/
+//    int _lastcol = 0;
+//    int _lastline = 0;
+    /******** Add by ut001000 renfeixiang 2020-07-16:增加 End***************/
 };
 
 }
