@@ -345,7 +345,7 @@ bool KeyboardTranslatorReader::decodeSequence(const QString& text,
                                               KeyboardTranslator::States& flagMask)
 {
     bool isWanted = true;
-    bool endOfItem = false;
+    //bool endOfItem = false;
     QString buffer;
 
     Qt::KeyboardModifiers tempModifiers = modifiers;
@@ -358,7 +358,7 @@ bool KeyboardTranslatorReader::decodeSequence(const QString& text,
         const QChar& ch = text[i];
         bool isFirstLetter = i == 0;
         bool isLastLetter = ( i == text.count()-1 );
-        endOfItem = true;
+        bool endOfItem = true;
         if ( ch.isLetterOrNumber() )
         {
             endOfItem = false;
