@@ -26,11 +26,12 @@
 // Qt
 #include <QRect>
 #include <QTextStream>
+#include <QBitArray>
 #include <QVarLengthArray>
+#include <QSet>
 
 // Konsole
 #include "Character.h"
-#include "History.h"
 
 #define MODE_Origin    0
 #define MODE_Wrap      1
@@ -38,12 +39,17 @@
 #define MODE_Screen    3
 #define MODE_Cursor    4
 #define MODE_NewLine   5
-#define MODES_SCREEN   6
+#define MODE_AppScreen 6
+#define MODES_SCREEN   7
 
 namespace Konsole
 {
 
 class TerminalCharacterDecoder;
+class TerminalDisplay;
+class HistoryType;
+class HistoryScroll;
+class EscapeSequenceUrlExtractor;
 
 /**
     \brief An image of characters with associated attributes.
@@ -71,6 +77,24 @@ class TerminalCharacterDecoder;
 class Screen
 {
 public:
+    /* PlainText: Return plain text (default)
+     * ConvertToHtml: Specifies if returned text should have HTML tags.
+     * PreserveLineBreaks: Specifies whether new line characters should be
+     *      inserted into the returned text at the end of each terminal line.
+     * TrimLeadingWhitespace: Specifies whether leading spaces should be
+     *      trimmed in the returned text.
+     * TrimTrailingWhitespace: Specifies whether trailing spaces should be
+     *      trimmed in the returned text.
+     */
+    enum DecodingOption {
+        PlainText = 0x0,
+        ConvertToHtml = 0x1,
+        PreserveLineBreaks = 0x2,
+        TrimLeadingWhitespace = 0x4,
+        TrimTrailingWhitespace = 0x8
+    };
+    Q_DECLARE_FLAGS(DecodingOptions, DecodingOption)
+
     /** Construct a new screen image of size @p lines by @p columns. */
     Screen(int lines, int columns);
     ~Screen();
@@ -346,7 +370,7 @@ public:
      * is inserted at the current cursor position, otherwise it will replace the
      * character already at the current cursor position.
      */
-    void displayCharacter(wchar_t c);
+    void displayCharacter(uint c);
 
     // Do composition with last shown character FIXME: Not implemented yet for KDE 4
     void compose(const QString& compose);
@@ -386,10 +410,10 @@ public:
 
     /** Return the number of lines. */
     int getLines() const
-    { return lines; }
+    { return _lines; }
     /** Return the number of columns. */
     int getColumns() const
-    { return columns; }
+    { return _columns; }
     /** Return the number of lines in the history buffer. */
     int getHistLines() const;
     /**
@@ -453,7 +477,15 @@ public:
      * @param preserveLineBreaks Specifies whether new line characters should
      * be inserted into the returned text at the end of each terminal line.
      */
-    QString selectedText(bool preserveLineBreaks) const;
+    QString selectedText(const DecodingOptions options) const;
+
+    /**
+     * Convenience method.  Returns the text between two indices.
+     * @param startIndex Specifies the starting text index
+     * @param endIndex Specifies the ending text index
+     * @param options See Screen::DecodingOptions
+     */
+    QString text(int startIndex, int endIndex, const DecodingOptions options) const;
 
     /**
      * Copies part of the output to a stream.
@@ -474,8 +506,7 @@ public:
      * @param preserveLineBreaks Specifies whether new line characters should
      * be inserted into the returned text at the end of each terminal line.
      */
-    void writeSelectionToStream(TerminalCharacterDecoder* decoder , bool
-                                preserveLineBreaks = true) const;
+    void writeSelectionToStream(TerminalCharacterDecoder* decoder, const DecodingOptions options) const;
 
     /**
      * Checks if the text between from and to is inside the current
@@ -554,7 +585,49 @@ public:
       */
     static void fillWithDefaultChar(Character* dest, int count);
 
+    void setCurrentTerminalDisplay(TerminalDisplay *display)
+    {
+        _currentTerminalDisplay = display;
+    }
+
+    TerminalDisplay *currentTerminalDisplay()
+    {
+        return _currentTerminalDisplay;
+    }
+
+    QSet<uint> usedExtendedChars() const
+    {
+        QSet<uint> result;
+        for (int i = 0; i < _lines; ++i) {
+            const ImageLine &il = _screenLines[i];
+            for (int j = 0; j < il.length(); ++j) {
+                if (il[j].rendition & RE_EXTENDED_CHAR) {
+                    result << il[j].character;
+                }
+            }
+        }
+        return result;
+    }
+
+    static const Character DefaultChar;
+
+    // Return the total number of lines before resize (fix scroll glitch)
+    int getOldTotalLines();
+
+    // Return if it was a resize signal (fix scroll glitch)
+    bool isResize();
+
+    // Set reflow condition
+    void setReflowLines(bool enable);
+
+    // 设置sessionId
+    void setSessionId(int sessionId);
+
 private:
+    Screen(const Screen &) = delete;
+    Screen &operator=(const Screen &) = delete;
+
+    EscapeSequenceUrlExtractor *urlExtractor() const;
 
     //copies a line of text from the screen or history into a stream using a
     //specified character decoder.  Returns the number of lines actually copied,
@@ -572,12 +645,12 @@ private:
                           int count,
                           TerminalCharacterDecoder* decoder,
                           bool appendNewLine,
-                          bool preserveLineBreaks) const;
+                          const DecodingOptions options) const;
 
     //fills a section of the screen image with the character 'c'
     //the parameters are specified as offsets from the start of the screen image.
     //the loc(x,y) macro can be used to generate these values from a column,line pair.
-    void clearImage(int loca, int loce, char c);
+    void clearImage(int loca, int loce, char c, bool resetLineRendition = true);
 
     //move screen image between 'sourceBegin' and 'sourceEnd' to 'dest'.
     //the parameters are specified as offsets from the start of the screen image.
@@ -591,6 +664,8 @@ private:
     void scrollDown(int from, int i);
 
     void addHistLine();
+    // add lines from screen to history and remove from screen the added lines (used to resize lines and columns)
+    void fastAddHistLine();
 
     void initTabStops();
 
@@ -601,7 +676,7 @@ private:
     // copies text from 'startIndex' to 'endIndex' to a stream
     // startIndex and endIndex are positions generated using the loc(x,y) macro
     void writeToStream(TerminalCharacterDecoder* decoder, int startIndex,
-                       int endIndex, bool preserveLineBreaks = true) const;
+                       int endIndex, const DecodingOptions options) const;
     // copies 'count' lines from the screen buffer into 'dest',
     // starting from 'startLine', where 0 is the first line in the screen buffer
     void copyFromScreen(Character* dest, int startLine, int count) const;
@@ -609,55 +684,75 @@ private:
     // starting from 'startLine', where 0 is the first line in the history
     void copyFromHistory(Character* dest, int startLine, int count) const;
 
+    // returns a buffer that can hold at most 'count' characters,
+    // where the number of reallocations and object reinitializations
+    // should be as minimal as possible
+    static Character *getCharacterBuffer(const int size);
+
+    // Get the cursor line after checking if its app mode or not
+    int getCursorLine();
+    // Set the cursor line after checking if its app mode or not
+    void setCursorLine (int newLine);
+
+    int getLineLength(const int line) const;
+
+    // returns the width in columns of the specified screen line,
+    // taking DECDWL/DECDHL (double width/height modes) into account.
+    int getScreenLineColumns(const int line) const;
 
     // screen image ----------------
-    int lines;
-    int columns;
+    int _lines;
+    int _columns;
 
     typedef QVector<Character> ImageLine;      // [0..columns]
-    ImageLine*          screenLines;    // [lines]
+    QVector<ImageLine> _screenLines;           // [lines]
+    int _screenLinesSize;                      // _screenLines.size()
 
     int _scrolledLines;
     QRect _lastScrolledRegion;
 
     int _droppedLines;
 
-    QVarLengthArray<LineProperty,64> lineProperties;
+    int _oldTotalLines;
+    bool _isResize;
+    bool _enableReflowLines = true;//自动换行功能，默认为true
+
+    QVarLengthArray<LineProperty,64> _lineProperties;
 
     // history buffer ---------------
-    HistoryScroll* history;
+    HistoryScroll* _history;
 
     // cursor location
-    int cuX;
-    int cuY;
+    int _cuX;
+    int _cuY;
 
     // cursor color and rendition info
-    CharacterColor currentForeground;
-    CharacterColor currentBackground;
-    quint8 currentRendition;
+    CharacterColor _currentForeground;
+    CharacterColor _currentBackground;
+    quint8 _currentRendition;
 
     // margins ----------------
     int _topMargin;
     int _bottomMargin;
 
     // states ----------------
-    bool currentModes[MODES_SCREEN];
-    bool savedModes[MODES_SCREEN];
+    bool _currentModes[MODES_SCREEN];
+    bool _savedModes[MODES_SCREEN];
 
     // ----------------------------
 
-    QBitArray tabStops;
+    QBitArray _tabStops;
 
     // selection -------------------
-    int selBegin; // The first location selected.
-    int selTopLeft;    // TopLeft Location.
-    int selBottomRight;    // Bottom Right Location.
-    bool blockSelectionMode;  // Column selection mode
+    int _selBegin; // The first location selected.
+    int _selTopLeft;    // TopLeft Location.
+    int _selBottomRight;    // Bottom Right Location.
+    bool _blockSelectionMode;  // Column selection mode
 
     // effective colors and rendition ------------
-    CharacterColor effectiveForeground; // These are derived from
-    CharacterColor effectiveBackground; // the cu_* variables above
-    quint8 effectiveRendition;          // to speed up operation
+    CharacterColor _effectiveForeground; // These are derived from
+    CharacterColor _effectiveBackground; // the cu_* variables above
+    quint8 _effectiveRendition;          // to speed up operation
 
     class SavedState
     {
@@ -671,15 +766,24 @@ private:
         CharacterColor foreground;
         CharacterColor background;
     };
-    SavedState savedState;
+    SavedState _savedState;
 
     // last position where we added a character
-    int lastPos;
+    int _lastPos;
 
     // used in REP (repeating char)
-    unsigned short lastDrawnChar;
+    quint32 _lastDrawnChar;
+    EscapeSequenceUrlExtractor *_escapeSequenceUrlExtractor;
+
+    //when we handle scroll commands, we need to know which screenwindow will scroll
+    TerminalDisplay *_currentTerminalDisplay;
+
+    //用于标识Session
+    int _sessionId;
+
+    // Vt102Emulation defined max argument value that can be passed to a Screen function
+    const int MAX_SCREEN_ARGUMENT = 40960;
 
-    static Character defaultChar;
 };
 
 }
