@@ -1,10 +1,12 @@
+// Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #include "customcommandoptdlg.h"
 #include "termcommandlinkbutton.h"
-#include "shortcutmanager.h"
-#include "shortcutmanager.h"
 #include "utils.h"
-#include"service.h"
-//#include "mainwindow.h"
+#include "service.h"
 
 #include <DButtonBox>
 #include <DPushButton>
@@ -18,8 +20,6 @@
 #include <DWidgetUtil>
 #include <DLog>
 #include <DFontSizeManager>
-#include <DVerticalLine>
-#include <DGuiApplicationHelper>
 
 #include <QGridLayout>
 #include <QHBoxLayout>
@@ -27,17 +27,29 @@
 #include <QApplication>
 #include <QClipboard>
 
-CustomCommandOptDlg::CustomCommandOptDlg(CustomCmdOptType type, CustomCommandItemData *currItemData, QWidget *parent)
-    : DAbstractDialog(parent),
-      m_type(type),
-      m_nameLineEdit(new DLineEdit),
-      m_commandLineEdit(new DLineEdit),
-      m_shortCutLineEdit(new DKeySequenceEdit),
-      m_bDelOpt(false)
+#ifdef QT_DEBUG
+Q_LOGGING_CATEGORY(customcommand,"org.deepin.terminal.customcommand")
+#else
+Q_LOGGING_CATEGORY(customcommand,"org.deepin.terminal.customcommand",QtInfoMsg)
+#endif
+
+CustomCommandOptDlg::CustomCommandOptDlg(CustomCmdOptType type, CustomCommandData *currItemData, QWidget *parent)
+    : DAbstractDialog(parent)
+    , m_type(type)
+    , m_nameLineEdit(new DLineEdit(this))
+    , m_commandLineEdit(new DLineEdit(this))
+    , m_shortCutLineEdit(new DKeySequenceEdit(this))
+    , m_bDelOpt(false)
 {
+    /******** Add by ut001000 renfeixiang 2020-08-13:增加 Begin***************/
+    Utils::set_Object_Name(this);
+    m_nameLineEdit->setObjectName("CustomNameLineEdit");
+    m_commandLineEdit->setObjectName("CustomCommandLineEdit");
+    m_shortCutLineEdit->setObjectName("CustomShortCutLineEdit");
+    /******** Add by ut001000 renfeixiang 2020-08-13:增加 End***************/
     setWindowModality(Qt::WindowModal);
     if (currItemData) {
-        m_currItemData = new CustomCommandItemData;
+        m_currItemData = new CustomCommandData;
         *m_currItemData = *currItemData;
     }
 
@@ -56,79 +68,35 @@ CustomCommandOptDlg::~CustomCommandOptDlg()
     }
     if (m_newAction) {
         delete  m_newAction;
+        m_newAction = nullptr;
     }
 }
 
-void CustomCommandOptDlg::slotRefreshData(QString oldCmdName, QString newCmdName)
-{
-    if (m_type == CCT_ADD) {
-        return;
-    }
-    if (oldCmdName == "" && newCmdName == "") {
-        return;
-    }
-    if (m_currItemData->m_cmdName != oldCmdName) {
-        return;
-    }
-    m_bRefreshCheck = true;
-    qDebug() << "slotRefreshData---" <<  m_nameLineEdit->text();
-
-    QAction *currAction = new QAction(ShortcutManager::instance());
-    if (currAction == nullptr) {
-        qDebug() << "slotRefreshData---new QAction error!!!";
-        close();
-    }
-    currAction->setText(newCmdName);
-    QAction *existAction = nullptr;
-    existAction = ShortcutManager::instance()->checkActionIsExist(*currAction);
-    if (existAction == nullptr) {
-        delete currAction;
-        if (m_dlgDelete && m_dlgDelete->isVisible()) {
-            m_dlgDelete->close();
-            m_dlgDelete = nullptr;
-        }
-
-        close();
-    } else {
-        delete currAction;
-        m_nameLineEdit->setText(existAction->text());
-        m_commandLineEdit->setText(existAction->data().toString());
-        m_shortCutLineEdit->setKeySequence(existAction->shortcut());
-        m_currItemData->m_cmdName = newCmdName;
-    }
-
-}
-
-void CustomCommandOptDlg::closeRefreshDataConnection()
-{
-    disconnect(Service::instance(), &Service::refreshCommandPanel, this, &CustomCommandOptDlg::slotRefreshData);
-}
-
 void CustomCommandOptDlg::initUI()
 {
-    QWidget *contentFrame = new QWidget;
+    QWidget *contentFrame = new QWidget(this);
 
     QVBoxLayout *contentLayout = new QVBoxLayout;
-    contentLayout->setSpacing(10);
-    contentLayout->setContentsMargins(0, 0, 0, 0);
+    contentLayout->setSpacing(m_iSpaceSizeTen);
+    contentLayout->setContentsMargins(m_iLayoutSizeZero, m_iLayoutSizeZero, m_iLayoutSizeZero, m_iLayoutSizeZero);
 
-    QWidget *nameFrame = new QWidget;
-    nameFrame->setFixedWidth(459);
+    QWidget *nameFrame = new QWidget(this);
+    nameFrame->setFixedWidth(m_iFixedWidth);
     QHBoxLayout *nameLayout = new QHBoxLayout;
-    nameLayout->setSpacing(0);
-    nameLayout->setContentsMargins(28, 0, 30, 0);
+    nameLayout->setSpacing(m_iSpaceSizeZero);
+    nameLayout->setContentsMargins(m_iLayoutLeftSize, m_iLayoutSizeZero, m_iLayoutRightSize, m_iLayoutSizeZero);
     nameFrame->setLayout(nameLayout);
 
-    QWidget *cmdFrame = new QWidget;
+    QWidget *cmdFrame = new QWidget(this);
     QHBoxLayout *cmdLayout = new QHBoxLayout;
-    cmdLayout->setSpacing(0);
-    cmdLayout->setContentsMargins(28, 0, 30, 0);
+    cmdLayout->setSpacing(m_iSpaceSizeZero);
+    cmdLayout->setContentsMargins(m_iLayoutLeftSize, m_iLayoutSizeZero, m_iLayoutRightSize, m_iLayoutSizeZero);
     cmdFrame->setLayout(cmdLayout);
 
-    QWidget *scFrame = new QWidget;
+    QWidget *scFrame = new QWidget(this);
     QHBoxLayout *scLayout = new QHBoxLayout;
-    scLayout->setSpacing(0);
-    scLayout->setContentsMargins(28, 0, 30, 0);
+    scLayout->setSpacing(m_iSpaceSizeZero);
+    scLayout->setContentsMargins(m_iLayoutLeftSize, m_iLayoutSizeZero, m_iLayoutRightSize, m_iLayoutSizeZero);
     scFrame->setLayout(scLayout);
 
     DLabel *nameLabel = new DLabel(tr("Name:"));
@@ -137,9 +105,14 @@ void CustomCommandOptDlg::initUI()
 
     DLabel *shortCutLabel = new DLabel(tr("Shortcuts:"));
 
-    m_nameLineEdit->setFixedWidth(285);
-    m_commandLineEdit->setFixedWidth(285);
-    m_shortCutLineEdit->setFixedWidth(285);
+    m_nameLineEdit->setFixedWidth(m_iLineEditWidth);
+    m_commandLineEdit->setFixedWidth(m_iLineEditWidth);
+    m_shortCutLineEdit->setFixedWidth(m_iLineEditWidth);
+    m_shortCutLineEdit->setFocusPolicy(Qt::FocusPolicy::StrongFocus);
+
+    QTimer::singleShot(m_iSingleShotTime, this, [&]() {
+        m_nameLineEdit->lineEdit()->selectAll();
+    });
 
     m_nameLineEdit->lineEdit()->setPlaceholderText(tr("Required"));
     m_commandLineEdit->lineEdit()->setPlaceholderText(tr("Required"));
@@ -160,46 +133,40 @@ void CustomCommandOptDlg::initUI()
 
     contentFrame->setLayout(contentLayout);
 
-    connect(m_nameLineEdit, &DLineEdit::editingFinished, this, [ = ] {
-        if (m_nameLineEdit->text().isEmpty())
-        {
-            m_nameLineEdit->lineEdit()->setPlaceholderText(tr("Required"));
-        }
-    });
+    connect(m_nameLineEdit, &DLineEdit::editingFinished, this, &CustomCommandOptDlg::slotNameLineEditingFinished);
 
-    connect(m_commandLineEdit, &DLineEdit::editingFinished, this, [ = ] {
-        if (m_commandLineEdit->text().isEmpty())
-        {
-            m_commandLineEdit->lineEdit()->setPlaceholderText(tr("Required"));
-        }
-    });
+    connect(m_commandLineEdit, &DLineEdit::editingFinished, this, &CustomCommandOptDlg::slotCommandLineEditingFinished);
 
     addContent(contentFrame);
-    if (m_type == CCT_ADD) {
-        setFixedSize(459, 262);
+    //判断是添加操作窗口还是修改操作窗口
+    if (CCT_ADD == m_type) {
         setTitle(tr("Add Command"));
         initCommandFromClipBoardText();
 
-        getMainLayout()->addSpacing(18);
+#ifdef DTKWIDGET_CLASS_DSizeMode
+        getMainLayout()->addSpacing(m_iSpaceSizeSeven);
+#else
+        getMainLayout()->addSpacing(m_iSpaceSizeEighteen);
+#endif
     } else {
-        setFixedSize(459, 296);
         setTitle(tr("Edit Command"));
 
-        QWidget *deleteCmdWidget = new QWidget;
-        deleteCmdWidget->setFixedHeight(54);
+        m_deleteCmdWidget = new QWidget(this);
+        m_deleteCmdWidget->setFixedHeight(m_iFixedHeight);
 
         QHBoxLayout *deleteCmdLayout = new QHBoxLayout();
-        deleteCmdLayout->setSpacing(0);
-        deleteCmdLayout->setContentsMargins(0, 0, 0, 0);
+        deleteCmdLayout->setSpacing(m_iSpaceSizeZero);
+        deleteCmdLayout->setContentsMargins(m_iLayoutSizeZero, m_iLayoutSizeZero, m_iLayoutSizeZero, m_iLayoutSizeZero);
         TermCommandLinkButton *deleteCmdBtn = new TermCommandLinkButton();
+        deleteCmdBtn->setFocusPolicy(Qt::TabFocus);
         deleteCmdBtn->setText(tr("Delete Command"));
 
         deleteCmdLayout->addStretch();
         deleteCmdLayout->addWidget(deleteCmdBtn);
         deleteCmdLayout->addStretch();
-        deleteCmdWidget->setLayout(deleteCmdLayout);
+        m_deleteCmdWidget->setLayout(deleteCmdLayout);
 
-        getMainLayout()->addWidget(deleteCmdWidget);
+        getMainLayout()->addWidget(m_deleteCmdWidget);
 
         connect(deleteCmdBtn, &DCommandLinkButton::clicked, this, &CustomCommandOptDlg::slotDelCurCustomCommand);
 
@@ -210,75 +177,187 @@ void CustomCommandOptDlg::initUI()
         m_nameLineEdit->setText(strName);
         m_commandLineEdit->setText(strCommad);
         m_shortCutLineEdit->setKeySequence(keyseq);
-
-#ifdef UI_DEBUG
-        deleteCmdWidget->setStyleSheet("background:green");
-#endif
     }
 
     addCancelConfirmButtons();
-    setCancelBtnText(tr("Cancel"));
-    if (m_type == CCT_ADD) {
-        setConfirmBtnText(tr("Add"));
+    setCancelBtnText(tr("Cancel", "button"));
+    if (CCT_ADD == m_type) {
+        setConfirmBtnText(tr("Add", "button"));
     } else {
-        setConfirmBtnText(tr("Save"));
+        setConfirmBtnText(tr("Save", "button"));
     }
     m_lastCmdShortcut = m_shortCutLineEdit->keySequence().toString();
     connect(this, &CustomCommandOptDlg::confirmBtnClicked, this, &CustomCommandOptDlg::slotAddSaveButtonClicked);
-    connect(m_shortCutLineEdit, &KeySequenceEdit::editingFinished, this, [ = ](const QKeySequence & sequence) {
-        //删除
-        if (sequence.toString() == "Backspace") {
-            m_shortCutLineEdit->clear();
-            m_lastCmdShortcut = "";
-            return ;
+    connect(m_shortCutLineEdit, &KeySequenceEdit::editingFinished, this, &CustomCommandOptDlg::slotShortCutLineEditingFinished);
+
+#ifdef DTKWIDGET_CLASS_DSizeMode
+    updateSizeMode();
+    connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::sizeModeChanged, this, &CustomCommandOptDlg::updateSizeMode);
+    // 字体变更时更新布局
+    connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::fontChanged, this, [this](){
+        if (isVisible() && layout()) {
+            layout()->invalidate();
+            updateGeometry();
+            // 根据新界面布局，刷新界面大小
+            QTimer::singleShot(0, this, [=](){ resize(SETTING_DIALOG_WIDTH, minimumSizeHint().height()); });
         }
-        // 取消
-        if (sequence.toString() == "Esc") {
-            m_shortCutLineEdit->clear();
-            m_shortCutLineEdit->setKeySequence(QKeySequence(m_lastCmdShortcut));
-            /***add by ut001121 zhangmeng 20200521 在快捷键编辑框中按下ESC键时退出窗口 修复BUG27554***/
-            close();
-            return ;
-        }
-        QString checkName;
-        //QString seq = m_shortCutLineEdit->text();
-        if (m_type != CCT_ADD) {
-            checkName = m_nameLineEdit->text();
-        }
-        QString reason;
-        // 判断快捷键是否冲突
-        if (!ShortcutManager::instance()->checkShortcutValid(checkName, sequence.toString(), reason)) {
-            // 冲突
-            if (sequence.toString() != "Esc") {
-                showShortcutConflictMsgbox(reason);
-            }
-            m_shortCutLineEdit->clear();
-            m_shortCutLineEdit->setKeySequence(QKeySequence(m_lastCmdShortcut));
-            return;
-        }
-        m_lastCmdShortcut = sequence.toString();
     });
 
-#ifdef UI_DEBUG
-    contentFrame->setStyleSheet("background:cyan");
-    nameLabel->setStyleSheet("background:#0000ff");
-    cmdLabel->setStyleSheet("background:#00ff00");
-    shortCutLabel->setStyleSheet("background:yellow");
-
-    m_nameLineEdit->setStyleSheet("background:#ff00ff");
-    m_commandLineEdit->setStyleSheet("background:#ff00ff");
-    m_shortCutLineEdit->setStyleSheet("background:#ff00ff");
+#else
+    if (CCT_ADD == m_type) {
+        setFixedSize(m_iFixedWidth, m_iFixedHeightAddSize);
+    } else {
+        setFixedSize(m_iFixedWidth, m_iFixedHeightEditSize);
+    }
 #endif
 }
 
-// fix bug 18366 终端中选中内容，创建自定义命令，选中内容没有自动粘贴到命令输入框
+inline void CustomCommandOptDlg::slotNameLineEditingFinished()
+{
+    if (m_nameLineEdit->text().isEmpty())
+        m_nameLineEdit->lineEdit()->setPlaceholderText(tr("Required"));
+}
+
+inline void CustomCommandOptDlg::slotCommandLineEditingFinished()
+{
+    if (m_commandLineEdit->text().isEmpty())
+        m_commandLineEdit->lineEdit()->setPlaceholderText(tr("Required"));
+}
+
+inline void CustomCommandOptDlg::slotShortCutLineEditingFinished(const QKeySequence &sequence)
+{
+    //删除
+    if ("Backspace" == sequence.toString()) {
+        qCInfo(customcommand) << "The KeySequenceE is Backspace";
+        m_shortCutLineEdit->clear();
+        m_lastCmdShortcut = "";
+        return;
+    }
+    // 取消
+    if ("Esc" == sequence.toString()) {
+        qCInfo(customcommand) << "The KeySequenceE is Esc";
+        m_shortCutLineEdit->clear();
+        m_shortCutLineEdit->setKeySequence(QKeySequence(m_lastCmdShortcut));
+        /***add by ut001121 zhangmeng 20200521 在快捷键编辑框中按下ESC键时退出窗口 修复BUG27554***/
+        close();
+        return;
+    }
+    QString checkName;
+    if (m_type != CCT_ADD)
+        checkName = m_nameLineEdit->text();
+
+    QString reason;
+    // 判断快捷键是否冲突
+    if (!ShortcutManager::instance()->checkShortcutValid(checkName, sequence.toString(), reason)) {
+        qCWarning(customcommand) << "Shortcut key conflict";
+        // 冲突
+        if (sequence.toString() != "Esc") {
+            qCWarning(customcommand) << "The current shortcut key is not Esc! ("<< sequence <<")";
+            showShortcutConflictMsgbox(reason);
+        }
+        m_shortCutLineEdit->clear();
+        m_shortCutLineEdit->setKeySequence(QKeySequence(m_lastCmdShortcut));
+        return;
+    }
+    m_lastCmdShortcut = sequence.toString();
+}
+
+void CustomCommandOptDlg::initUITitle()
+{
+    QVBoxLayout *mainLayout = new QVBoxLayout();
+    mainLayout->setSpacing(SPACEHEIGHT);
+    mainLayout->setContentsMargins(0, 0, 0, 10);
+
+    QHBoxLayout *titleLayout = new QHBoxLayout();
+    titleLayout->setSpacing(0);
+    titleLayout->setContentsMargins(0, 0, 0, 0);
+
+    m_titleBar = new QWidget(this);
+    m_titleBar->setObjectName("CustomTitleBar");//Add by ut001000 renfeixiang 2020-08-13
+    m_titleBar->setFixedHeight(50);
+    m_titleBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+    m_titleBar->setLayout(titleLayout);
+
+    m_logoIcon = new DLabel(this);
+    m_logoIcon->setObjectName("CustomLogoIcon");//Add by ut001000 renfeixiang 2020-08-13
+    m_logoIcon->setFixedSize(QSize(50, 50));
+    m_logoIcon->setFocusPolicy(Qt::NoFocus);
+    m_logoIcon->setAttribute(Qt::WA_TransparentForMouseEvents);
+
+    m_closeButton = new DWindowCloseButton(this);
+    m_closeButton->setObjectName("CustomCloseButton");//Add by ut001000 renfeixiang 2020-08-13
+    m_closeButton->setFocusPolicy(Qt::TabFocus);//m_closeButton->setFocusPolicy(Qt::NoFocus);
+    m_closeButton->setIconSize(QSize(50, 50));
+
+    m_titleText = new DLabel(this);
+    m_titleText->setObjectName("CustomTitleTextLabel");//Add by ut001000 renfeixiang 2020-08-13
+    m_titleText->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+    m_titleText->setAlignment(Qt::AlignCenter);
+    DFontSizeManager::instance()->bind(m_titleText, DFontSizeManager::T5, QFont::DemiBold);
+    // 字色
+    DPalette palette = m_titleText->palette();
+    QColor color;
+    if (DApplicationHelper::DarkType == DApplicationHelper::instance()->themeType())
+        color = QColor::fromRgb(192, 198, 212, 255);
+    else
+        color = QColor::fromRgb(0, 26, 46, 255);
+
+    palette.setBrush(QPalette::WindowText, color);
+    m_titleText->setPalette(palette);
+
+    titleLayout->addWidget(m_logoIcon, 0, Qt::AlignLeft | Qt::AlignVCenter);
+    titleLayout->addWidget(m_titleText, 0, Qt::AlignHCenter);
+    titleLayout->addWidget(m_closeButton, 0, Qt::AlignRight | Qt::AlignTop);
+
+    //Dialog content
+    m_contentLayout = new QVBoxLayout();
+    m_contentLayout->setObjectName("CustomContentLayout");//Add by ut001000 renfeixiang 2020-08-13
+    m_contentLayout->setSpacing(0);
+    m_contentLayout->setContentsMargins(0, 0, 0, 0);
+
+    m_content = new QWidget(this);
+    m_content->setObjectName("CustomContentWidget");//Add by ut001000 renfeixiang 2020-08-13
+    m_content->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    m_content->setLayout(m_contentLayout);
+
+    mainLayout->addWidget(m_titleBar, 0, Qt::AlignTop);
+    mainLayout->addWidget(m_content);
+    setLayout(mainLayout);
+
+    m_mainLayout = mainLayout;
+}
+
+void CustomCommandOptDlg::initTitleConnections()
+{
+    connect(m_closeButton, &DWindowCloseButton::clicked, this, &CustomCommandOptDlg::slotCloseButtonClicked);
+    // 字体颜色随主题变化变化
+    connect(DApplicationHelper::instance(), &DGuiApplicationHelper::themeTypeChanged, this, &CustomCommandOptDlg::slotThemeTypeChanged);
+}
+
+inline void CustomCommandOptDlg::slotCloseButtonClicked()
+{
+    this->close();
+}
+
+inline void CustomCommandOptDlg::slotThemeTypeChanged(DGuiApplicationHelper::ColorType themeType)
+{
+    DPalette palette = m_titleText->palette();
+    QColor color;
+    if (DApplicationHelper::DarkType == themeType)
+        color = QColor::fromRgb(192, 198, 212, 255);
+    else
+        color = QColor::fromRgb(0, 26, 46, 255);
+
+    palette.setBrush(QPalette::WindowText, color);
+    m_titleText->setPalette(palette);
+}
+
 void CustomCommandOptDlg::initCommandFromClipBoardText()
 {
     if (m_commandLineEdit) {
-        // QString clipText = ShortcutManager::instance()->getClipboardCommandData();
         MainWindow *main = Utils::getMainWindow(this);//;getMainWindow();
         if (main != nullptr) {
-            QString clipText = main->selectedText(true);
+            QString clipText = main->selectedText();
             m_commandLineEdit->setText(clipText.trimmed());
         }
     }
@@ -296,29 +375,33 @@ bool CustomCommandOptDlg::isDelCurCommand()
 
 void CustomCommandOptDlg::slotAddSaveButtonClicked()
 {
-    qDebug() <<  __FUNCTION__ << __LINE__;
     QString strName = m_nameLineEdit->text();
     QString strCommand = m_commandLineEdit->text();
     QKeySequence keytmp = m_shortCutLineEdit->keySequence();
 
+    strName = strName.trimmed();//空格的名称是无效的，剔除名称前后的空格
     if (strName.isEmpty()) {
+        qCWarning(customcommand) << "The name of the user-defined command is empty!";
         m_nameLineEdit->showAlertMessage(tr("Please enter a name"), m_nameLineEdit->parentWidget());
         return;
     }
 
     /***add begin by ut001121 zhangmeng 20200615 限制名称字符长度 修复BUG31286***/
     if (strName.length() > MAX_NAME_LEN) {
+        qCWarning(customcommand) << "The name should be no more than 32 characters";
         m_nameLineEdit->showAlertMessage(QObject::tr("The name should be no more than 32 characters"), m_nameLineEdit->parentWidget());
         return;
     }
     /***add end by ut001121***/
-
-    if (strCommand.isEmpty()) {
+    QString strCommandtemp = strCommand.trimmed();//空格的命令是无效的
+    if (strCommandtemp.isEmpty()) {
+        qCWarning(customcommand) << "The custom command is empty";
         m_commandLineEdit->showAlertMessage(tr("Please enter a command"), m_commandLineEdit->parentWidget());
         return;
     }
 
     m_newAction = new QAction(ShortcutManager::instance());
+    m_newAction->setObjectName("CustomQAction");//Add by ut001000 renfeixiang 2020-08-13
     m_newAction->setText(strName);
     m_newAction->setData(strCommand);
     m_newAction->setShortcut(m_shortCutLineEdit->keySequence());
@@ -326,22 +409,21 @@ void CustomCommandOptDlg::slotAddSaveButtonClicked()
     /************************ Mod by m000743 sunchengxi 2020-04-21:自定义命令修改的异常问题 Begin************************/
     QAction *existAction = nullptr;
     int icount = 0;
-    if (m_type == CCT_MODIFY) {
-
-        //if (m_bRefreshCheck   && (!checkSequence(m_shortCutLineEdit->keySequence()))) {
-        //    return;
-        //}
-
+    if (CCT_MODIFY == m_type) {
+        qCInfo(customcommand) << "It is the modify type of custom command operation";
         if (m_bRefreshCheck) {
+            qCInfo(customcommand) << "CustomCommand was refreshed";
             QAction *refreshExitAction = nullptr;
             refreshExitAction = ShortcutManager::instance()->checkActionIsExist(*m_newAction);
             if (refreshExitAction) {
+                qCWarning(customcommand) << "The custom command already exists";
                 accept();
                 return;
             }
         }
 
         if (strName == m_currItemData->m_cmdName && strCommand == m_currItemData->m_cmdText && keytmp == QKeySequence(m_currItemData->m_cmdShortcut)) {
+            qCInfo(customcommand) << "The custom command don't need to save again.";
             accept();
             return;
         }
@@ -349,20 +431,22 @@ void CustomCommandOptDlg::slotAddSaveButtonClicked()
         existAction = ShortcutManager::instance()->checkActionIsExistForModify(*m_newAction);
 
         if (strName != m_currItemData->m_cmdName) {
+            qCInfo(customcommand) << "Custom commands have been changed.";
             QList<QAction *> &customCommandActionList = ShortcutManager::instance()->getCustomCommandActionList();
             for (int i = 0; i < customCommandActionList.size(); i++) {
                 QAction *curAction = customCommandActionList[i];
                 QString strCmdName = curAction->text();
-                if (strCmdName == strName) {
+                if (strCmdName == strName)
                     icount++;
-                }
             }
         }
     } else {
+        qCInfo(customcommand) << "It is the add type of custom command operation";
         existAction = ShortcutManager::instance()->checkActionIsExist(*m_newAction);
     }
 
     if (nullptr != existAction || icount) {
+        qCInfo(customcommand) << "The name already exists";
         QString strFistLine = tr("The name already exists,");
         QString strSecondeLine = tr("please input another one.");
         Utils::showSameNameDialog(this, strFistLine, strSecondeLine);
@@ -376,17 +460,16 @@ bool CustomCommandOptDlg::checkSequence(const QKeySequence &sequence)
 {
     QString checkName = m_nameLineEdit->text();
 
-    if (sequence.toString() == "") {
+    if (sequence.toString().isEmpty())
         return true;
-    }
 
     QString reason;
-// 判断快捷键是否冲突
+    // 判断快捷键是否冲突
     if (!ShortcutManager::instance()->checkShortcutValid(checkName, sequence.toString(), reason)) {
         // 冲突
-        if (sequence.toString() != "Esc") {
+        if (sequence.toString() != "Esc")
             showShortcutConflictMsgbox(reason);
-        }
+
         m_shortCutLineEdit->clear();
         m_shortCutLineEdit->setKeySequence(QKeySequence(m_lastCmdShortcut));
 
@@ -399,7 +482,7 @@ bool CustomCommandOptDlg::checkSequence(const QKeySequence &sequence)
     return true;
 }
 
-void CustomCommandOptDlg::setModelIndex(QModelIndex mi)
+void CustomCommandOptDlg::setModelIndex(const QModelIndex &mi)
 {
     modelIndex = mi;
 }
@@ -410,66 +493,6 @@ void CustomCommandOptDlg::slotDelCurCustomCommand()
     reject();
 }
 
-void CustomCommandOptDlg::initUITitle()
-{
-    QVBoxLayout *mainLayout = new QVBoxLayout();
-    mainLayout->setSpacing(0);
-    mainLayout->setContentsMargins(0, 0, 0, 10);
-
-    QHBoxLayout *titleLayout = new QHBoxLayout();
-    titleLayout->setSpacing(0);
-    titleLayout->setContentsMargins(0, 0, 0, 0);
-
-    m_titleBar = new QWidget(this);
-    m_titleBar->setFixedHeight(50);
-    m_titleBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-    m_titleBar->setLayout(titleLayout);
-
-    m_logoIcon = new DLabel(this);
-    m_logoIcon->setFixedSize(QSize(50, 50));
-    m_logoIcon->setFocusPolicy(Qt::NoFocus);
-    m_logoIcon->setAttribute(Qt::WA_TransparentForMouseEvents);
-
-    m_closeButton = new DWindowCloseButton(this);
-    m_closeButton->setFocusPolicy(Qt::NoFocus);
-    m_closeButton->setIconSize(QSize(50, 50));
-
-    m_titleText = new DLabel(this);
-    m_titleText->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
-    m_titleText->setAlignment(Qt::AlignCenter);
-    DFontSizeManager::instance()->bind(m_titleText, DFontSizeManager::T5, QFont::DemiBold);
-    // 字色
-    DPalette palette = m_titleText->palette();
-//    palette.setColor(QPalette::WindowText, palette.color(DPalette::TextTitle));
-    QColor color;
-    if (DApplicationHelper::instance()->themeType() == DApplicationHelper::DarkType) {
-        color = QColor::fromRgb(192, 198, 212, 255);
-    } else {
-        color = QColor::fromRgb(0, 26, 46, 255);
-    }
-    palette.setBrush(QPalette::WindowText, color);
-    m_titleText->setPalette(palette);
-
-    titleLayout->addWidget(m_logoIcon, 0, Qt::AlignLeft | Qt::AlignVCenter);
-    titleLayout->addWidget(m_titleText, 0, Qt::AlignHCenter);
-    titleLayout->addWidget(m_closeButton, 0, Qt::AlignRight | Qt::AlignTop);
-
-    //Dialog content
-    m_contentLayout = new QVBoxLayout();
-    m_contentLayout->setSpacing(0);
-    m_contentLayout->setContentsMargins(0, 0, 0, 0);
-
-    m_content = new QWidget(this);
-    m_content->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-    m_content->setLayout(m_contentLayout);
-
-    mainLayout->addWidget(m_titleBar, 0, Qt::AlignTop);
-    mainLayout->addWidget(m_content);
-    setLayout(mainLayout);
-
-    m_mainLayout = mainLayout;
-}
-
 void CustomCommandOptDlg::addCancelConfirmButtons()
 {
     QHBoxLayout *buttonsLayout = new QHBoxLayout();
@@ -478,104 +501,95 @@ void CustomCommandOptDlg::addCancelConfirmButtons()
 
     QFont btnFont;
     m_cancelBtn = new DPushButton(this);
+    m_cancelBtn->setObjectName("CustomCancelButton");//Add by ut001000 renfeixiang 2020-08-13
     m_cancelBtn->setFixedWidth(209);
-    m_cancelBtn->setFixedHeight(36);
     m_cancelBtn->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
     m_cancelBtn->setFont(btnFont);
 
     m_confirmBtn = new DSuggestButton(this);
+    m_confirmBtn->setObjectName("CustomConfirmButton");//Add by ut001000 renfeixiang 2020-08-13
     m_confirmBtn->setFixedWidth(209);
-    m_confirmBtn->setFixedHeight(36);
     m_confirmBtn->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
     m_confirmBtn->setFont(btnFont);
 
     //设置回车键默认响应的按钮
     m_confirmBtn->setDefault(true);
 
-    DVerticalLine *verticalLine = new DVerticalLine(this);
-    DPalette pa = DApplicationHelper::instance()->palette(verticalLine);
+    setTabOrder(m_confirmBtn, m_closeButton);//设置右上角关闭按钮的tab键控制顺序
+
+    m_verticalLine = new DVerticalLine(this);
+    DPalette pa = DApplicationHelper::instance()->palette(m_verticalLine);
     QColor splitColor = pa.color(DPalette::ItemBackground);
     pa.setBrush(DPalette::Background, splitColor);
-    verticalLine->setPalette(pa);
-    verticalLine->setBackgroundRole(QPalette::Background);
-    verticalLine->setAutoFillBackground(true);
-    verticalLine->setFixedSize(3, 28);
+    m_verticalLine->setPalette(pa);
+    m_verticalLine->setBackgroundRole(QPalette::Background);
+    m_verticalLine->setAutoFillBackground(true);
+    m_verticalLine->setFixedSize(3, 28);
 
     buttonsLayout->addWidget(m_cancelBtn);
-    buttonsLayout->addWidget(verticalLine);
+    buttonsLayout->addWidget(m_verticalLine);
     buttonsLayout->addWidget(m_confirmBtn);
     /************************ Add by m000743 sunchengxi 2020-04-15:默认enter回车按下，走确认校验流程 Begin************************/
     m_confirmBtn->setDefault(true);
     /************************ Add by m000743 sunchengxi 2020-04-15:默认enter回车按下，走确认校验流程 End ************************/
 
-    connect(m_cancelBtn, &DPushButton::clicked, this, [ = ]() {
-        qDebug() << "cancelBtnClicked";
-        m_confirmResultCode = QDialog::Rejected;
-        reject();
-        close();
-    });
+    connect(m_cancelBtn, &DPushButton::clicked, this, &CustomCommandOptDlg::slotCancelBtnClicked);
 
-    connect(m_confirmBtn, &DPushButton::clicked, this, [ = ]() {
-        qDebug() << "confirmBtnClicked";
-        m_confirmResultCode = QDialog::Accepted;
-        emit confirmBtnClicked();
-    });
+    connect(m_confirmBtn, &DPushButton::clicked, this, &CustomCommandOptDlg::slotConfirmBtnClicked);
 
     /************************ Add by m000743 sunchengxi 2020-04-21:快捷键编辑结束后，enter按下可以响应校验 Begin************************/
-    connect(m_shortCutLineEdit, &DKeySequenceEdit::editingFinished, this, [ = ]() {
-        qDebug() << "shourtCut editingFinished";
-        /******** Add by nt001000 renfeixiang 2020-05-14:修改快捷框输入后，添加（m_confirmBtn）按钮高亮问题,将光标从添加按钮取消，设置到快捷框上 Begin***************/
-        //m_confirmBtn->setFocus();
-        m_shortCutLineEdit->setFocus();
-        /******** Add by nt001000 renfeixiang 2020-05-14:修改快捷框输入后，添加（m_confirmBtn）按钮高亮问题,将光标从添加按钮取消，设置到快捷框上 End***************/
-    });
+    connect(m_shortCutLineEdit, &DKeySequenceEdit::editingFinished, this, &CustomCommandOptDlg::slotShortCutLineEditFinished);
     /************************ Add by m000743 sunchengxi 2020-04-21:快捷键编辑结束后，enter按下可以响应校验 End  ************************/
 
     m_mainLayout->addLayout(buttonsLayout);
 }
 
+inline void CustomCommandOptDlg::slotCancelBtnClicked()
+{
+    m_confirmResultCode = QDialog::Rejected;
+    reject();
+}
+
+inline void CustomCommandOptDlg::slotConfirmBtnClicked()
+{
+    m_confirmResultCode = QDialog::Accepted;
+    emit confirmBtnClicked();
+}
+
+inline void CustomCommandOptDlg::slotShortCutLineEditFinished()
+{
+    /******** Add by nt001000 renfeixiang 2020-05-14:修改快捷框输入后，添加（m_confirmBtn）按钮高亮问题,将光标从添加按钮取消，设置到快捷框上 Begin***************/
+    m_shortCutLineEdit->setFocus();
+    /******** Add by nt001000 renfeixiang 2020-05-14:修改快捷框输入后，添加（m_confirmBtn）按钮高亮问题,将光标从添加按钮取消，设置到快捷框上 End***************/
+}
+
 QVBoxLayout *CustomCommandOptDlg::getMainLayout()
 {
     return m_mainLayout;
 }
 
-void CustomCommandOptDlg::initTitleConnections()
-{
-    connect(m_closeButton, &DWindowCloseButton::clicked, this, [this]() {
-        this->close();
-    });
-    // 字体颜色随主题变化变化
-    connect(DApplicationHelper::instance(), &DGuiApplicationHelper::themeTypeChanged, m_titleText, [ = ](DGuiApplicationHelper::ColorType themeType) {
-        DPalette palette = m_titleText->palette();
-        //palette.setBrush(QPalette::WindowText, palette.color(DPalette::TextTitle));
-        QColor color;
-        if (themeType == DApplicationHelper::DarkType) {
-            color = QColor::fromRgb(192, 198, 212, 255);
-        } else {
-            color = QColor::fromRgb(0, 26, 46, 255);
-        }
-        palette.setBrush(QPalette::WindowText, color);
-        m_titleText->setPalette(palette);
-    });
-}
-
 QDialog::DialogCode CustomCommandOptDlg::getConfirmResult()
 {
     return m_confirmResultCode;
 }
 
-void CustomCommandOptDlg::setLogoVisable(bool visible)
-{
-    if (nullptr != m_logoIcon) {
-        m_logoIcon->setVisible(visible);
-    }
-}
+/*******************************************************************************
+ 1. @函数:    setLogoVisable
+ 2. @作者:    sunchengxi
+ 3. @日期:    2020-07-31
+ 4. @说明:    设置自定义命令操作窗口的图标是否可见
+*******************************************************************************/
+//void CustomCommandOptDlg::setLogoVisable(bool visible)
+//{
+//    if (nullptr != m_logoIcon) {
+//        m_logoIcon->setVisible(visible);
+//    }
+//}
 
 void CustomCommandOptDlg::setTitle(const QString &title)
 {
-    if (nullptr != m_titleText) {
+    if (nullptr != m_titleText)
         m_titleText->setText(title);
-    }
 }
 
 QLayout *CustomCommandOptDlg::getContentLayout()
@@ -595,46 +609,85 @@ void CustomCommandOptDlg::setConfirmBtnText(const QString &strConfirm)
     Utils::setSpaceInWord(m_confirmBtn);
 }
 
-/*******************************************************************************
- 1. @函数:    showShortcutConflictMsgbox
- 2. @作者:    ut000610 戴正文
- 3. @日期:    2020-05-21
- 4. @说明:    显示快捷键冲突弹窗
-*******************************************************************************/
 void CustomCommandOptDlg::showShortcutConflictMsgbox(QString txt)
 {
     /******** Modify by ut000610 daizhengwen 2020-05-27: 出现提示和快捷键显示不一致的问题 bug#28507****************/
-    if (txt.contains("Return")) {
-        txt.replace("Return", "Enter");
+    // fix#bug 37399
+    for (QString key : ShortcutManager::instance()->m_mapReplaceText.keys()) {
+        if (txt.contains(key))
+            txt.replace(key, ShortcutManager::instance()->m_mapReplaceText[key]);
     }
     /********************* Modify by ut000610 daizhengwen End ************************/
     // 若没有弹窗，初始化
     if (nullptr == m_shortcutConflictDialog) {
         m_shortcutConflictDialog = new DDialog(this);
+        m_shortcutConflictDialog->setObjectName("CustomShortcutConflictDialog");//Add by ut001000 renfeixiang 2020-08-13
         /******** Modify by nt001000 renfeixiang 2020-05-29:修改 因为弹框改为非模态之后，快捷框冲突选中快捷框功能移动这 Begin***************/
-        connect(m_shortcutConflictDialog, &DDialog::finished, m_shortcutConflictDialog, [this]() {
-            m_shortcutConflictDialog->hide();
-            /******** Add by nt001000 renfeixiang 2020-05-14:快捷框输入已经存在的快捷后，快捷框依然是选中状态 Begin***************/
-            QTimer::singleShot(30, this, [&]() {
-                m_shortCutLineEdit->setFocus();
-            });
-            /******** Add by nt001000 renfeixiang 2020-05-14:快捷框输入已经存在的快捷后，快捷框依然是选中状态 End***************/
-        });
+        connect(m_shortcutConflictDialog, &DDialog::finished, this, &CustomCommandOptDlg::slotShortcutConflictDialogFinished);
         /******** Modify by nt001000 renfeixiang 2020-05-29:修改 修改因为弹框改为非模态之后，快捷框冲突选中快捷框功能移动这 End***************/
         m_shortcutConflictDialog->setIcon(QIcon::fromTheme("dialog-warning"));
         /***mod by ut001121 zhangmeng 20200521 将确认按钮设置为默认按钮 修复BUG26960***/
-        m_shortcutConflictDialog->addButton(QString(tr("OK")), true, DDialog::ButtonNormal);
+        m_shortcutConflictDialog->addButton(QString(tr("OK", "button")), true, DDialog::ButtonNormal);
     }
     // 存在后显示
     m_shortcutConflictDialog->setTitle(QString(txt + QObject::tr("please set another one.")));
     m_shortcutConflictDialog->show();
 }
 
+inline void CustomCommandOptDlg::slotShortcutConflictDialogFinished()
+{
+    m_shortcutConflictDialog->hide();
+    /******** Add by nt001000 renfeixiang 2020-05-14:快捷框输入已经存在的快捷后，快捷框依然是选中状态 Begin***************/
+    QTimer::singleShot(30, this, &CustomCommandOptDlg::slotSetShortCutLineEditFocus);
+    /******** Add by nt001000 renfeixiang 2020-05-14:快捷框输入已经存在的快捷后，快捷框依然是选中状态 End***************/
+}
+
+inline void CustomCommandOptDlg::slotSetShortCutLineEditFocus()
+{
+    m_shortCutLineEdit->setFocus();
+}
+
+/**
+ * @brief 接收 DGuiApplicationHelper::sizeModeChanged() 信号, 根据不同的布局模式调整
+ *      当前界面的布局. 只能在界面创建完成后调用.
+ */
+void CustomCommandOptDlg::updateSizeMode()
+{
+#ifdef DTKWIDGET_CLASS_DSizeMode
+    if (DGuiApplicationHelper::isCompactMode()) {
+        m_titleBar->setFixedHeight(WIN_TITLE_BAR_HEIGHT_COMPACT);
+        m_logoIcon->setFixedSize(QSize(ICONSIZE_40_COMPACT, ICONSIZE_40_COMPACT));
+        m_closeButton->setIconSize(QSize(ICONSIZE_40_COMPACT, ICONSIZE_40_COMPACT));
+        m_verticalLine->setFixedSize(VERTICAL_WIDTH_COMPACT, VERTICAL_HEIGHT_COMPACT);
+        if (m_deleteCmdWidget) {
+            m_deleteCmdWidget->setFixedHeight(m_iFixedHeightCompact);
+        }
+
+    } else {
+        m_titleBar->setFixedHeight(WIN_TITLE_BAR_HEIGHT);
+        m_logoIcon->setFixedSize(QSize(ICONSIZE_50, ICONSIZE_50));
+        m_closeButton->setIconSize(QSize(ICONSIZE_50, ICONSIZE_50));
+        m_verticalLine->setFixedSize(VERTICAL_WIDTH, VERTICAL_HEIGHT);
+        if (m_deleteCmdWidget) {
+            m_deleteCmdWidget->setFixedHeight(m_iFixedHeight);
+        }
+    }
+
+    if (layout()) {
+        layout()->invalidate();
+    }
+
+    updateGeometry();
+    // 根据新界面布局，刷新界面大小
+    QTimer::singleShot(0, this, [=](){ resize(SETTING_DIALOG_WIDTH, minimumSizeHint().height()); });
+#endif
+}
+
 void CustomCommandOptDlg::addContent(QWidget *content)
 {
-    Q_ASSERT(nullptr != getContentLayout());
+    Q_ASSERT(nullptr != m_contentLayout);
 
-    getContentLayout()->addWidget(content);
+    m_contentLayout->addWidget(content);
 }
 
 void CustomCommandOptDlg::setIconPixmap(const QPixmap &iconPixmap)
@@ -653,3 +706,50 @@ void CustomCommandOptDlg::closeEvent(QCloseEvent *event)
 
     Q_EMIT closed();
 }
+
+void CustomCommandOptDlg::slotRefreshData(QString oldCmdName, QString newCmdName)
+{
+    if (CCT_ADD == m_type) {
+        qCWarning(customcommand) << "Currently is the add operation interface";
+        return;
+    }
+    //不进行刷新操作
+    if (oldCmdName.isEmpty() && newCmdName.isEmpty())
+        return;
+
+    //当前的自定义命令的名称 不是被修改的自定义名称时，不进行刷新操作
+    if (m_currItemData->m_cmdName != oldCmdName) {
+        return;
+    }
+    m_bRefreshCheck = true;
+    qCInfo(customcommand) << "Refresh custom command data.Curren command name is " <<  m_nameLineEdit->text();
+
+    QAction *currAction = new QAction(ShortcutManager::instance());
+    currAction->setText(newCmdName);
+    QAction *existAction = nullptr;
+    existAction = ShortcutManager::instance()->checkActionIsExist(*currAction);
+    //根据 newCmdName 查找自定义命令
+    if (nullptr == existAction) {
+        //不存在，则认为当前自定义命令在其他编辑窗口中删除了，如果打开了删除确认界面，则关闭删除确认界面。关闭当前编辑窗口
+        delete currAction;
+        if (m_dlgDelete && m_dlgDelete->isVisible()) {
+            m_dlgDelete->close();
+            m_dlgDelete = nullptr;
+        }
+
+        close();
+    } else {
+        //当前自定义命令在其他窗口修改，则更新数据
+        delete currAction;
+        m_nameLineEdit->setText(existAction->text());
+        m_commandLineEdit->setText(existAction->data().toString());
+        m_shortCutLineEdit->setKeySequence(existAction->shortcut());
+        m_currItemData->m_cmdName = newCmdName;
+    }
+
+}
+
+void CustomCommandOptDlg::closeRefreshDataConnection()
+{
+    disconnect(Service::instance(), &Service::refreshCommandPanel, this, &CustomCommandOptDlg::slotRefreshData);
+}
