@@ -1,3 +1,8 @@
+// Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #include "termproperties.h"
 
 TermProperties::TermProperties(QString workingDir)
@@ -11,9 +16,8 @@ TermProperties::TermProperties(QString workingDir, QString colorScheme)
     m_properties.insert(ColorScheme, colorScheme);
 }
 
-TermProperties::TermProperties(QMap<TermProperty, QVariant> list)
+TermProperties::TermProperties(QMap<TermProperty, QVariant> list) : m_properties(list)
 {
-    m_properties = list;
 }
 
 void TermProperties::setWorkingDir(QString workingDir)
