@@ -1,68 +1,97 @@
+// Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #include "termwidgetpage.h"
 #include "termwidget.h"
 #include "settings.h"
 #include "utils.h"
 #include "service.h"
 #include "windowsmanager.h"
+#include "mainwindow.h"
+#include "define.h"
 
 #include <DLog>
 #include <DDialog>
 
-#include <QUuid>
 #include <QVBoxLayout>
 #include <QApplication>
+#include <QLoggingCategory>
 
-TermWidgetPage::TermWidgetPage(TermProperties properties, QWidget *parent)
+Q_DECLARE_LOGGING_CATEGORY(views)
+TermWidgetPage::TermWidgetPage(const TermProperties &properties, QWidget *parent)
     : QWidget(parent), m_findBar(new PageSearchBar(this))
 {
-    //qDebug() << "parentTermWidgetPage" << parentWidget();
-    m_MainWindow = static_cast<MainWindow *>(parentWidget());
+    Utils::set_Object_Name(this);
+    m_MainWindow = qobject_cast<MainWindow *>(parentWidget());
     setFocusPolicy(Qt::NoFocus);
     setProperty("TAB_CUSTOM_NAME_PROPERTY", false);
+
+    //fix bug 55961 UOS SP2 OEM版本的系统下，一个窗口同时打开几个终端，各终端简无法切换
+    //Qt自带API接口QUuid::createUuid()在浪潮服务器上会生成重复的id：ffffffff-ffff-4fff-bfff-ffffffffffff
+    QString pageId = QString::number(reinterpret_cast<qulonglong>(this));
     // 生成唯一 pageID
-    setProperty("TAB_IDENTIFIER_PROPERTY", QUuid::createUuid().toString());
+    setProperty("TAB_IDENTIFIER_PROPERTY", pageId);
 
     TermWidget *w = createTerm(properties);
     m_layout = new QVBoxLayout(this);
+    m_layout->setObjectName("TermPageLayout");//Add by ut001000 renfeixiang 2020-08-13
     m_layout->setSpacing(0);
     m_layout->setContentsMargins(0, 0, 0, 0);
     m_layout->addWidget(w);
     setLayout(m_layout);
-    //qDebug() << "w->parent()" << w->parent();
 
     // Init find bar.
-    m_findBar->move(this->x() - 100, this->y() - 100);
     connect(m_findBar, &PageSearchBar::findNext, this, &TermWidgetPage::handleFindNext);
     connect(m_findBar, &PageSearchBar::findPrev, this, &TermWidgetPage::handleFindPrev);
-    connect(m_findBar, &PageSearchBar::keywordChanged, this, [ = ](QString keyword) {
-        handleUpdateSearchKeyword(keyword);
-    });
-    connect(DApplicationHelper::instance(), &DApplicationHelper::themeTypeChanged, this, [ = ]() {
-        applyTheme();
-    });
+    connect(m_findBar, &PageSearchBar::keywordChanged, this, &TermWidgetPage::handleKeywordChanged);
+    connect(DApplicationHelper::instance(), &DApplicationHelper::themeTypeChanged, this, &TermWidgetPage::handleThemeTypeChanged);
 
     /******** Modify by nt001000 renfeixiang 2020-05-27:修改 增加参数区别remove和purge卸载命令 2020-05-28 优化代码 Begin***************/
-    connect(this, &TermWidgetPage::uninstallTerminal, this, [this](QString commandname) {
-        //MainWindow *mainWindow = qobject_cast<MainWindow *>(parent);
-        //构造函数中已经获取了mainwindow窗口，无需在获取一遍
-        if (m_MainWindow->hasRunningProcesses()) {
-            if (!Utils::showExitUninstallConfirmDialog()) {
-                return false;
-            }
-        }
-        return Utils::showUnistallConfirmDialog(commandname);
-    });
+    connect(this, &TermWidgetPage::uninstallTerminal, this, &TermWidgetPage::handleUninstallTerminal);
     /******** Modify by nt001000 renfeixiang 2020-05-27:修改 增加参数区别remove和purge卸载命令 Begin***************/
 
     m_currentTerm = w;
+
+#ifdef DTKWIDGET_CLASS_DSizeMode
+    connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::sizeModeChanged, this, [this](){
+        m_findBar->move(width() - m_findBar->width(), 0);
+    }, Qt::QueuedConnection);
+#endif
+}
+
+inline void TermWidgetPage::handleKeywordChanged(const QString &keyword)
+{
+    handleUpdateSearchKeyword(keyword);
+}
+
+inline void TermWidgetPage::handleThemeTypeChanged()
+{
+    applyTheme();
+}
+
+inline bool TermWidgetPage::handleUninstallTerminal(QString commandname)
+{
+    //MainWindow *mainWindow = qobject_cast<MainWindow *>(parent);
+    //构造函数中已经获取了mainwindow窗口，无需在获取一遍
+    if (m_MainWindow && m_MainWindow->hasRunningProcesses()) {
+        if (!Utils::showExitUninstallConfirmDialog())
+            return false;
+    }
+    return Utils::showUninstallConfirmDialog(commandname);
 }
 
 MainWindow *TermWidgetPage::parentMainWindow()
 {
-    //qDebug()<<"parentMainWindow" <<parentWidget();
     return m_MainWindow;
 }
 
+void TermWidgetPage::setParentMainWindow(MainWindow *mainWin)
+{
+    m_MainWindow = mainWin;
+}
+
 void TermWidgetPage::setSplitStyle(DSplitter *splitter)
 {
     splitter->setHandleWidth(1);
@@ -70,7 +99,9 @@ void TermWidgetPage::setSplitStyle(DSplitter *splitter)
 
     if (handle) {
         //分割线颜色暂时设置为Highlight颜色，需要和UI确认下
-        DPalette pa = DApplicationHelper::instance()->palette(handle);
+        //此处代码暂时保留  //DPalette pa = DApplicationHelper::instance()->palette(handle);
+        //bug#57044 中的分割线颜色，保留的代码对默认主题，和十个内置主题的颜色是正确获取，但是在自定义的颜色获取存在异常，采取如下方式获取
+        DPalette pa = DApplicationHelper::instance()->applicationPalette();
         QColor splitBrush = pa.color(DPalette::Highlight);
         pa.setBrush(DPalette::Background, splitBrush);
         handle->setPalette(pa);
@@ -84,46 +115,47 @@ TermWidget *TermWidgetPage::currentTerminal()
     return m_currentTerm;
 }
 
-//TermWidget *TermWidgetPage::split(Qt::Orientation orientation)
-//{
-//    return split(currentTerminal(), orientation);
-//}
-/*******************************************************************************
- 1. @函数:
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-05-14
- 4. @说明:
-*******************************************************************************/
+QString TermWidgetPage::getCurrentTerminalTitle()
+{
+    if (nullptr == m_currentTerm)
+        return "";
+    return m_currentTerm->getTabTitle();
+}
+
 void TermWidgetPage::split(Qt::Orientation orientation)
 {
     parentMainWindow()->showPlugin(MainWindow::PLUGIN_TYPE_NONE);
     TermWidget *term = m_currentTerm;
-    if (getTerminalCount() == 1) {
-        qDebug() << "first split";
+    if (1 == getTerminalCount()) {
+        qCInfo(views) << "first split";
         QSplitter *firstSplit = createSubSplit(term, orientation);
         m_layout->addWidget(firstSplit);
-        return ;
+        //return ;
+    } else {
+        qCInfo(views) << "not first split";
+        QSplitter *upSplit = qobject_cast<QSplitter *>(term->parent());
+        int index = upSplit->indexOf(term);
+        QList<int> parentSizes = upSplit->sizes();
+
+        // 用新的Split分割布局替换原来的位置
+        QSplitter *subSplit = createSubSplit(term, orientation);
+        upSplit->insertWidget(index, subSplit);
+        upSplit->setSizes(parentSizes);
+        setSplitStyle(upSplit);
     }
 
-    qDebug() << "not first split";
-    QSplitter *upSplit = qobject_cast<QSplitter *>(term->parent());
-    int index = upSplit->indexOf(term);
-    QList<int> parentSizes = upSplit->sizes();
+    /******** Add by ut001000 renfeixiang 2020-08-07:新增分屏时改变大小，bug#41436***************/
+    parentMainWindow()->updateMinHeight();
 
-    // 用新的Split分割布局替换原来的位置
-    QSplitter *subSplit = createSubSplit(term, orientation);
-    upSplit->insertWidget(index, subSplit);
-    upSplit->setSizes(parentSizes);
-    setSplitStyle(upSplit);
+    //分屏时切换到当前选中主题方案
+    QString  expandThemeStr = "";
+    expandThemeStr = Settings::instance()->extendColorScheme();
+    if (!expandThemeStr.isEmpty())
+        emit DApplicationHelper::instance()->themeTypeChanged(DGuiApplicationHelper::instance()->themeType());
 
     return ;
 }
-/*******************************************************************************
- 1. @函数:    createSubSplit
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-05-14
- 4. @说明:    创建分屏
-*******************************************************************************/
+
 DSplitter *TermWidgetPage::createSubSplit(TermWidget *term, Qt::Orientation orientation)
 {
     TermProperties properties(term->workingDirectory());
@@ -133,23 +165,23 @@ DSplitter *TermWidgetPage::createSubSplit(TermWidget *term, Qt::Orientation orie
     // 意义与名称是相反的
     DSplitter *subSplit = new DSplitter(orientation == Qt::Horizontal ? Qt::Vertical : Qt::Horizontal,
                                         this);
+
     subSplit->setFocusPolicy(Qt::NoFocus);
     subSplit->insertWidget(0, term);
     subSplit->insertWidget(1, newTerm);
     subSplit->setSizes({ 1, 1 });
     setSplitStyle(subSplit);
     setCurrentTerminal(newTerm);
+    /******** Modify by ut000439 wangpeili 2020-07-27: fix bug 39371: 分屏线可以拉到边****/
+    subSplit->setChildrenCollapsible(false);
+    /********************* Modify by n014361 wangpeili End ************************/
+
     return subSplit;
 }
-/*******************************************************************************
- 1. @函数:
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-05-15
- 4. @说明:
-*******************************************************************************/
+
 void TermWidgetPage::closeSplit(TermWidget *term, bool hasConfirmed)
 {
-    qDebug() << "TermWidgetPage::closeSplit:" << term->getSessionId();
+    qCInfo(views) << "TermWidgetPage::closeSplit:" << term->getSessionId();
     if (getTerminalCount() > 1) {
         if (!hasConfirmed && term->hasRunningProcess()) {
             showExitConfirmDialog(Utils::CloseType_Terminal, 1, parentMainWindow());
@@ -169,21 +201,29 @@ void TermWidgetPage::closeSplit(TermWidget *term, bool hasConfirmed)
         }
         // 上级不是分屏控件，就是布局在控制了
         else {
-            qDebug() << "TermWidgetPage only one term exist!";
+            qCWarning(views) << "TermWidgetPage only one term exist!";
             m_layout->addWidget(brother);
         }
 
         // 子控件的变化会引起焦点的变化，控制焦点要放在最后
         if (nextTerm != nullptr) {
-            qDebug() << "nextTerm change" << m_currentTerm->getSessionId();
+            qCInfo(views) << "nextTerm change" << m_currentTerm->getSessionId();
             nextTerm->setFocus();
+        } else {
+            qCWarning(views) << "can not found nextTerm in TermWidget";
         }
 
-        // 释放控件
+        // 释放控件,并隐藏term、upSplit，避免出现闪现窗口bug#80809
+        term->hide();
         term->deleteLater();
+        // 断开相关的连接：(UT_MainWindow_Test, slotShortcutCloseWorkspace)出现的崩溃问题
+        Settings::instance()->disconnect(term);
+        upSplit->hide();
         upSplit->setParent(nullptr);
         upSplit->deleteLater();
-        qDebug() << "page terminal count =" << getTerminalCount();
+        qCInfo(views) << "page terminal count =" << getTerminalCount();
+        /******** Add by ut001000 renfeixiang 2020-08-07:关闭分屏时改变大小，bug#41436***************/
+        parentMainWindow()->updateMinHeight();
         return;
     }
     parentMainWindow()->closeTab(identifier());
@@ -219,7 +259,7 @@ void TermWidgetPage::showExitConfirmDialog(Utils::CloseType type, int count, QWi
 
 //    if (type == Utils::CloseType_Terminal) {
 //        connect(dlg, &DDialog::finished, this, [this](int result) {
-//            qDebug() << result;
+//            qCInfo(views) << result;
 //            // 有弹窗消失
 //            Service::instance()->setIsDialogShow(window(), false);
 //            if (result == 1) {
@@ -231,7 +271,7 @@ void TermWidgetPage::showExitConfirmDialog(Utils::CloseType type, int count, QWi
 
 //    if (type == Utils::CloseType_OtherTerminals) {
 //        connect(dlg, &DDialog::finished, this, [this](int result) {
-//            qDebug() << result;
+//            qCInfo(views) << result;
 //            // 有弹窗消失
 //            Service::instance()->setIsDialogShow(window(), false);
 //            if (result == 1) {
@@ -263,76 +303,24 @@ void TermWidgetPage::closeOtherTerminal(bool hasConfirmed)
     }
 
     QList<TermWidget *> termList = findChildren<TermWidget *>();
+    // 终端数量小于2,执行关闭其他窗口操作
+    if (termList.count() < 2) {
+        qCInfo(views) << "current window doesn't have other terminal, can't close other terminals.";
+        return;
+    }
 
     int currSessionId = m_currentTerm->getSessionId();
     //exit protection
     for (TermWidget *term : qAsConst(termList)) {
-        if (term->getSessionId() != currSessionId) {
+        if (term->getSessionId() != currSessionId)
             // 前面已经检测过了，不用重复检测
             closeSplit(term, true);
-        }
     }
 
     setTerminalOpacity(Settings::instance()->opacity());
+    m_MainWindow->focusCurrentPage();
 }
 
-#if 0
-// 待删除
-typedef struct {
-    QPoint topLeft;
-    QPoint middle;
-    QPoint bottomRight;
-} CoordinateRect;
-
-// 待删除
-static void transpose(QPoint *point)
-{
-    int x = point->x();
-    point->setX(point->y());
-    point->setY(x);
-}
-
-// 待删除
-static void transposeTransform(CoordinateRect *point)
-{
-    transpose(&point->topLeft);
-    transpose(&point->middle);
-    transpose(&point->bottomRight);
-}
-
-// 待删除
-static void flipTransform(CoordinateRect *point)
-{
-    QPoint oldTopLeft = point->topLeft;
-    point->topLeft = -(point->bottomRight);
-    point->bottomRight = -(oldTopLeft);
-    point->middle = -(point->middle);
-}
-
-// 待删除
-static void normalizeToRight(CoordinateRect *point, Qt::Edge dir)
-{
-    switch (dir) {
-    case Qt::LeftEdge:
-        flipTransform(point);
-        break;
-    case Qt::RightEdge:
-        // No-op
-        break;
-    case Qt::TopEdge:
-        flipTransform(point);
-        transposeTransform(point);
-        break;
-    case Qt::BottomEdge:
-        transposeTransform(point);
-        break;
-    default:
-        qFatal("Invalid navigation");
-        return;
-    }
-}
-#endif
-
 /******** Modify by ut000439 wangpeili 2020-01-07:  修改了计算上下左右判断的方法 ********×****/
 void TermWidgetPage::focusNavigation(Qt::Edge dir)
 {
@@ -342,48 +330,40 @@ void TermWidgetPage::focusNavigation(Qt::Edge dir)
     //QMap<TermWidget *, QRect> mapTermRect;
     for (TermWidget *term : qAsConst(termList)) {
         if (GetRect(term).contains(comparPoint)) {
-            qDebug() << "yes!" << comparPoint.x() << comparPoint.y();
             dst = term;
             break;
         }
     }
-    if (dst) {
+    if (dst)
         dst->setFocus();
-    }
 }
-/*******************************************************************************
- 1. @函数:    getTerminalCount
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-21
- 4. @说明:    获取当前页的终端个数
-*******************************************************************************/
+
 int TermWidgetPage::getTerminalCount()
 {
     return findChildren<TermWidget *>().size();
 }
-/********************* Modify by ut000439 wangpeili End ************************/
 
-/*******************************************************************************
- 1. @函数:  QRect TermWidgetPage::GetRect(TermWidgetWrapper *term)
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:  获取标签页内所有子窗体的区域
-*******************************************************************************/
+bool TermWidgetPage::hasHasHorizontalSplit()
+{
+    qCInfo(views) << "start hasHasHorizontalSplit";
+    QList<QSplitter *> splitList = findChildren<QSplitter *>();
+    for (QSplitter *split : splitList) {
+        if (Qt::Vertical == split->orientation()) {
+            return  true;
+        }
+    }
+    return  false;
+}
+/********************* Modify by ut000439 wangpeili End ************************/
 QRect TermWidgetPage::GetRect(TermWidget *term)
 {
     QPoint leftTop = term->mapTo(term->window(), QPoint(0, 0));
     QPoint rightBottom = term->mapTo(term->window(), QPoint(term->width(), term->height()));
     QRect rec(leftTop, rightBottom);
-    qDebug() << "leftTop: " << leftTop.x() << leftTop.y() << "rightBottom: " << rightBottom.x() << rightBottom.y();
+    qCInfo(views) << "leftTop: " << leftTop.x() << leftTop.y() << "rightBottom: " << rightBottom.x() << rightBottom.y();
     return rec;
 }
 
-/*******************************************************************************
- 1. @函数: QPoint TermWidgetPage::GetComparePoint(TermWidgetWrapper *term, Qt::Edge dir)
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-08
- 4. @说明:     获取窗口上下左右键需要判断的点位信息
-*******************************************************************************/
 QPoint TermWidgetPage::GetComparePoint(TermWidget *term, Qt::Edge dir)
 {
     QPoint leftTop = term->mapTo(term->window(), QPoint(0, 0));
@@ -412,9 +392,8 @@ int TermWidgetPage::runningTerminalCount()
     int count = 0;
     QList<TermWidget *> termList = findChildren<TermWidget *>();
     for (TermWidget *term : termList) {
-        if (term->hasRunningProcess()) {
+        if (term->hasRunningProcess())
             count++;
-        }
     }
     return count;
 }
@@ -424,9 +403,8 @@ TermProperties TermWidgetPage::createCurrentTerminalProperties()
     TermProperties properties;
 
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         properties[WorkingDir] = currentTerminal()->workingDirectory();
-    }
 
     return properties;
 }
@@ -434,23 +412,23 @@ TermProperties TermWidgetPage::createCurrentTerminalProperties()
 void TermWidgetPage::setTerminalOpacity(qreal opacity)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setTermOpacity(opacity);
-    }
 }
 
 void TermWidgetPage::setColorScheme(const QString &name)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setColorScheme(name);
-    }
 }
 
-void TermWidgetPage::sendTextToCurrentTerm(const QString &text)
+void TermWidgetPage::sendTextToCurrentTerm(const QString &text, bool isRemoteConnect)
 {
     TermWidget *term = currentTerminal();
     if (term) {
+        //标记当前命令是代码中通过sendText发给终端的(而不是用户手动输入的命令)
+        term->setProperty("isSendByRemoteManage", QVariant(isRemoteConnect));
         term->sendText(text);
     }
 }
@@ -458,206 +436,147 @@ void TermWidgetPage::sendTextToCurrentTerm(const QString &text)
 void TermWidgetPage::copyClipboard()
 {
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         term->copyClipboard();
-    }
 }
 
-/*******************************************************************************
- 1. @函数: void TermWidgetPage::pasteClipboard()
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明: 用户粘贴时候，直接粘贴剪切板内容
-*******************************************************************************/
 void TermWidgetPage::pasteClipboard()
 {
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         term->pasteClipboard();
-    }
-}
-
-void TermWidgetPage::toggleShowSearchBar()
-{
-    TermWidget *term = currentTerminal();
-    if (term) {
-        term->toggleShowSearchBar();
-    }
 }
 
 void TermWidgetPage::zoomInCurrentTierminal()
 {
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         term->zoomIn();
-    }
 }
 
 void TermWidgetPage::zoomOutCurrentTerminal()
 {
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         term->zoomOut();
-    }
 }
 
-/*******************************************************************************
- 1. @函数:   void TermWidgetPage::setFontSize(int fontSize)
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:    设置字体大小
-*******************************************************************************/
 void TermWidgetPage::setFontSize(int fontSize)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setTermFontSize(fontSize);
-    }
 }
 
-/*******************************************************************************
- 1. @函数:   void TermWidgetPage::setFont(QString fontName)
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:    设置字体
-*******************************************************************************/
 void TermWidgetPage::setFont(QString fontName)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setTermFont(fontName);
-    }
 }
 
-/*******************************************************************************
- 1. @函数:   void TermWidgetPage::selectAll()
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:    全选
-*******************************************************************************/
 void TermWidgetPage::selectAll()
 {
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         term->setSelectionAll();
-    }
 }
 
-/*******************************************************************************
- 1. @函数:   void TermWidgetPage::skipToNextCommand()
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:    跳转到下一命令
-*******************************************************************************/
-void TermWidgetPage::skipToNextCommand()
-{
-    TermWidget *term = currentTerminal();
-    if (term) {
-        term->skipToNextCommand();
-    }
-}
-
-/*******************************************************************************
- 1. @函数:   void TermWidgetPage::skipToPreCommand()
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:    跳转到前一命令
-*******************************************************************************/
-void TermWidgetPage::skipToPreCommand()
-{
-    TermWidget *term = currentTerminal();
-    if (term) {
-        term->skipToPreCommand();
-    }
-}
-
-/*******************************************************************************
- 1. @函数:   void TermWidgetPage::setcursorShape()
- 2. @作者:     ut000439 王培利
- 3. @日期:     2020-01-10
- 4. @说明:    设置鼠标形状
-*******************************************************************************/
 void TermWidgetPage::setcursorShape(int shape)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setCursorShape(shape);
-    }
 }
 
-/*******************************************************************************
- 1. @函数:    setBlinkingCursor(bool enable)
- 2. @作者:    王培利ut000439
- 3. @日期:    2020-01-10
- 4. @说明:    设置鼠标闪烁
-*******************************************************************************/
 void TermWidgetPage::setBlinkingCursor(bool enable)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setBlinkingCursor(enable);
-    }
 }
 
 void TermWidgetPage::setPressingScroll(bool enable)
 {
     QList<TermWidget *> termList = findChildren<TermWidget *>();
-    for (TermWidget *term : termList) {
+    for (TermWidget *term : termList)
         term->setPressingScroll(enable);
-    }
 }
-/*******************************************************************************
- 1. @函数:    showSearchBar(bool enable)
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-24
- 4. @说明:    是否显示搜索框
-*******************************************************************************/
-void TermWidgetPage::showSearchBar(bool enable)
+
+void TermWidgetPage::showSearchBar(int state)
 {
-    if (enable) {
+    /******** Modify by ut001000 renfeixiang 2020-08-28:修改bug 45227,SearchBar没有显示，且不需要显示时，return Begin***************/
+    // 沒显示，且不要显示
+    if (!m_findBar->isVisible() && state != SearchBar_Show)
+        return;
+    /******** Modify by ut001000 renfeixiang 2020-08-28***************/
+    if (SearchBar_Show == state) {
         /******** Add by nt001000 renfeixiang 2020-05-18:修改雷神窗口太小时，查询界面使用不方便，将雷神窗口变大适应正常的查询界面 Begin***************/
-        if (m_MainWindow->isQuakeMode() && m_MainWindow->height() < 220) {
-            m_MainWindow->resize(m_MainWindow->width(), 220); //首先设置雷神界面的大小
+        if (m_MainWindow->isQuakeMode() && m_MainWindow->height() < LISTMINHEIGHT) {
+            //因为拉伸函数设置了FixSize，导致自定义界面弹出时死循环，然后崩溃的问题
+            QuakeWindow *quakeWindow = qobject_cast<QuakeWindow *>(m_MainWindow);
+            if (nullptr == quakeWindow)
+                return;
+            quakeWindow->switchEnableResize(true);
+            m_MainWindow->resize(m_MainWindow->width(), LISTMINHEIGHT); //首先设置雷神界面的大小
             m_MainWindow->showPlugin(MainWindow::PLUGIN_TYPE_SEARCHBAR);//重新打开查询界面，当前流程结束
+            //窗口弹出后，重新判断雷神窗口是否需要有拉伸属性
+            quakeWindow->switchEnableResize();
             return;
         }
         /******** Add by nt001000 renfeixiang 2020-05-18:修改雷神窗口太小时，查询界面使用不方便，将雷神窗口变大适应正常的查询界面 End***************/
         m_findBar->raise();
         m_findBar->clearHoldContent();
         m_findBar->show();
-        m_findBar->move(width() - 382, 0);
+        //Add by ut001000 renfeixiang 2020-12-02 在搜索框弹出时，添加设置Term的m_bHasSelect为false函数
+        if (m_currentTerm != nullptr)
+            m_currentTerm->setNoHasSelect();
+        m_findBar->move(width() - m_findBar->width(), 0);
         QTimer::singleShot(10, this, [ = ] { m_findBar->focus(); });
-    } else {
+    } else if (SearchBar_Hide == state) {
         m_findBar->hide();
+    } else if (SearchBar_FocusOut == state) {
+        /******** Modify by ut001000 renfeixiang 2020-08-28:修改bug 45227,焦点只有在m_findBar上时，才将焦点设置到CurrentPage Begin***************/
+        MainWindow *w = Utils::getMainWindow(this);
+        if (w && w->isFocusOnList())
+            w->focusCurrentPage();
+        m_findBar->hide();
+        /******** Modify by ut001000 renfeixiang 2020-08-28 End***************/
     }
 }
 
-/*******************************************************************************
- 1. @函数:    showRenameTitleDialog
- 2. @作者:    ut000610 戴正文
- 3. @日期:    2020-05-21
- 4. @说明:    判断是否有重命名弹窗，有则显示，没有则创建
-*******************************************************************************/
-void TermWidgetPage::showRenameTitleDialog(QString oldTitle)
+void TermWidgetPage::showRenameTitleDialog()
 {
     // 弹窗显示
     Service::instance()->setIsDialogShow(window(), true);
+    if (nullptr == m_renameDlg) {
+        m_renameDlg = new TabRenameDlg(this);
+        m_renameDlg->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+        QString tabTitleFormat = currentTerminal()->getTabTitleFormat();
+        QString remoteTabTitleFormat = currentTerminal()->getRemoteTabTitleFormat();
+        m_renameDlg->setText(tabTitleFormat, remoteTabTitleFormat);
+        m_renameDlg->setFocusPolicy(Qt::NoFocus);
+        m_renameDlg->setAttribute(Qt::WA_DeleteOnClose);
 
-    if (nullptr == m_renameDialog) {
-        m_renameDialog = new TermInputDialog(this);
-        m_renameDialog->setAttribute(Qt::WA_DeleteOnClose);
-        connect(m_renameDialog, &TermInputDialog::finished, m_renameDialog, [ = ]() {
-            // 弹窗隐藏或消失
-            Service::instance()->setIsDialogShow(window(), false);
-            m_renameDialog = nullptr;
-        });
-        m_renameDialog->setFixedSize(380, 180);
-        m_renameDialog->setIcon(QIcon::fromTheme("deepin-terminal"));
-        m_renameDialog->setFocusPolicy(Qt::NoFocus);
-        m_renameDialog->showDialog(oldTitle, this);
+        //bug 53463 ut000442 判断当前是否为远程链接状态，根据状态设置对应行的选中
+        //设置全选效果
+        m_renameDlg->setFocusOnEdit(currentTerminal()->isConnectRemote());
+
+        m_renameDlg->open();
+
+        connect(m_renameDlg, &TabRenameDlg::finished, this, &TermWidgetPage::handleTabRenameDlgFinished);
+
+        connect(m_renameDlg, &TabRenameDlg::tabTitleFormatRename, this, &TermWidgetPage::tabTitleFormatChanged);
     }
 }
 
+inline void TermWidgetPage::handleTabRenameDlgFinished()
+{
+    // 弹窗隐藏或消失
+    Service::instance()->setIsDialogShow(window(), false);
+    m_renameDlg = nullptr;
+}
+
 void TermWidgetPage::onTermRequestRenameTab(QString newTabName)
 {
     if (newTabName.isEmpty()) {
@@ -671,10 +590,12 @@ void TermWidgetPage::onTermRequestRenameTab(QString newTabName)
     }
 }
 
-void TermWidgetPage::onTermTitleChanged(QString title) const
+void TermWidgetPage::onTermTitleChanged(QString title)
 {
     TermWidget *term = qobject_cast<TermWidget *>(sender());
-    if (m_currentTerm == term) {
+    // 标题内容没变化的话不发，不是当前终端改变，不发
+    if (term == m_currentTerm && m_tabTitle != title) {
+        m_tabTitle = title;
         emit termTitleChanged(title);
     }
 }
@@ -684,7 +605,6 @@ void TermWidgetPage::onTermGetFocus()
     TermWidget *term = qobject_cast<TermWidget *>(sender());
     setCurrentTerminal(term);
     emit Service::instance()->currentTermChange(m_currentTerm);
-    qDebug() << "onTermGetFocus" << m_currentTerm->getSessionId();
     emit termGetFocus();
 }
 
@@ -692,55 +612,50 @@ void TermWidgetPage::onTermClosed()
 {
     TermWidget *w = qobject_cast<TermWidget *>(sender());
     if (!w) {
-        qDebug() << "TermWidgetPage::onTermClosed: Unknown object to handle" << w;
-        Q_ASSERT(0);
+        qCWarning(views) << "TermWidgetPage::onTermClosed: Unknown object to handle" << w;
+        return;
     }
     closeSplit(w);
 }
-/*******************************************************************************
- 1. @函数:    handleFindNext
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-24
- 4. @说明:    查找下一个接口
-*******************************************************************************/
+
 void TermWidgetPage::handleFindNext()
 {
-    qDebug() << m_findBar->searchKeytxt();
     setMismatchAlert(false);
     m_currentTerm->search(m_findBar->searchKeytxt(), true, true);
 }
 
-/*******************************************************************************
- 1. @函数:    handleFindPrev
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-24
- 4. @说明:    查找上一个接口
-*******************************************************************************/
 void TermWidgetPage::handleFindPrev()
 {
     setMismatchAlert(false);
     m_currentTerm->search(m_findBar->searchKeytxt(), false, false);
 }
 
-void TermWidgetPage::slotFindbarClose()
-{
-    //    EditWrapper *wrapper = currentWrapper();
-    //    if (wrapper->bottomBar()->isHidden())
-    //    {
-    //        wrapper->bottomBar()->show();
-    //    }
-}
-
-void TermWidgetPage::handleRemoveSearchKeyword()
-{
-    //    currentWrapper()->textEditor()->removeKeywords();
-}
 /*******************************************************************************
- 1. @函数:    handleUpdateSearchKeyword
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-24
- 4. @说明:    更新key(没有输入回车情况),自动查找接口
+ 1. @函数:    slotFindbarClose
+ 2. @作者:    ut000439 wangpeili
+ 3. @日期:    2020-08-12
+ 4. @说明:    空函数，待处理
 *******************************************************************************/
+//void TermWidgetPage::slotFindbarClose()
+//{
+//    EditWrapper *wrapper = currentWrapper();
+//    if (wrapper->bottomBar()->isHidden())
+//    {
+//        wrapper->bottomBar()->show();
+//    }
+//}
+
+/*******************************************************************************
+ 1. @函数:    handleRemoveSearchKeyword
+ 2. @作者:    ut000439 wangpeili
+ 3. @日期:    2020-08-12
+ 4. @说明:    空函数，待处理
+*******************************************************************************/
+//void TermWidgetPage::handleRemoveSearchKeyword()
+//{
+//    currentWrapper()->textEditor()->removeKeywords();
+//}
+
 void TermWidgetPage::handleUpdateSearchKeyword(const QString &keyword)
 {
     setMismatchAlert(false);
@@ -751,12 +666,7 @@ void TermWidgetPage::handleUpdateSearchKeyword(const QString &keyword)
         // m_currentTerm->search(m_findBar->SearchKeytxt(), true, false);
     }
 }
-/*******************************************************************************
- 1. @函数:    applyTheme
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-03-011
- 4. @说明:    非DTK控件手动匹配系统主题的修改
-*******************************************************************************/
+
 void TermWidgetPage::applyTheme()
 {
     updateSplitStyle();
@@ -765,8 +675,36 @@ void TermWidgetPage::applyTheme()
 void TermWidgetPage::updateSplitStyle()
 {
     QList<DSplitter *> splitList = findChildren<DSplitter *>();
-    for (DSplitter *splitter : qAsConst(splitList)) {
+    for (DSplitter *splitter : qAsConst(splitList))
         setSplitStyle(splitter);
+}
+
+void TermWidgetPage::slotShowPluginChanged(const QString name)
+{
+    MainWindow *mainWindow = qobject_cast<MainWindow *>(sender());
+    // 判断是否是当前页，不是当前页不用管
+    if (mainWindow && mainWindow->currentPage() == this) {
+        // 显示和隐藏搜索框
+        if (MainWindow::PLUGIN_TYPE_SEARCHBAR == name) {
+            // 显示搜索框
+            this->showSearchBar(SearchBar_Show);
+        } else if (MainWindow::PLUGIN_TYPE_NONE == name) {
+            // 隐藏搜索框，焦点落回终端
+            this->showSearchBar(SearchBar_FocusOut);
+        } else {
+            // 隐藏搜索框
+            this->showSearchBar(SearchBar_Hide);
+        }
+    }
+}
+
+void TermWidgetPage::slotQuakeHidePlugin()
+{
+    MainWindow *mainWindow = qobject_cast<MainWindow *>(sender());
+    // 判断是否是当前页，不是当前页不用管
+    if (mainWindow && mainWindow->currentPage() == this) {
+        // 隐藏搜索框
+        this->showSearchBar(SearchBar_Hide);
     }
 }
 
@@ -775,12 +713,15 @@ void TermWidgetPage::setCurrentTerminal(TermWidget *term)
     TermWidget *oldTerm = m_currentTerm;
     m_currentTerm = term;
     if (oldTerm != m_currentTerm) {
-        qDebug() << "m_currentTerm change" << m_currentTerm->getSessionId();
-        if (m_currentTerm->isTitleChanged()) {
-            emit termTitleChanged(m_currentTerm->title());
-        } else {
-            emit termTitleChanged(windowTitle());
-        }
+        // 当前界面切换
+        qCInfo(views) << "Current terminal change" << m_currentTerm->getSessionId();
+        QString tabTitle = term->getTabTitle();
+        // 当前标签为空，标签格式不为空 => 未得到term参数，暂不上传数据
+        if ((tabTitle == DEFAULT_TAB_TITLE) && !term->getCurrentTabTitleFormat().trimmed().isEmpty())
+            return;
+        m_tabTitle = tabTitle;
+        // 当前窗口变化修改标签标题
+        emit termTitleChanged(m_tabTitle);
     }
 }
 
@@ -790,41 +731,34 @@ TermWidget *TermWidgetPage::createTerm(TermProperties properties)
     connect(term, &TermWidget::termRequestRenameTab, this, &TermWidgetPage::onTermRequestRenameTab);
     connect(term, &TermWidget::termTitleChanged, this, &TermWidgetPage::onTermTitleChanged);
     connect(term, &TermWidget::termGetFocus, this, &TermWidgetPage::onTermGetFocus);
-    connect(term, &TermWidget::leftMouseClick, this, [this]() {
-        parentMainWindow()->showPlugin(MainWindow::PLUGIN_TYPE_NONE);
-    });
+    connect(term, &TermWidget::leftMouseClick, this, &TermWidgetPage::handleLeftMouseClick);
 
     connect(term, &TermWidget::finished, this, &TermWidgetPage::onTermClosed);
-    qDebug() << "create Terminal, sessionId = " << term->getSessionId();
+    qCInfo(views) << "create Terminal, sessionId = " << term->getSessionId();
+    // 对标签页重命名设置
+    connect(this, &TermWidgetPage::tabTitleFormatChanged, term, &TermWidget::renameTabFormat);
     return term;
 }
 
+inline void TermWidgetPage::handleLeftMouseClick()
+{
+    parentMainWindow()->showPlugin(MainWindow::PLUGIN_TYPE_NONE);
+}
+
 void TermWidgetPage::setTextCodec(QTextCodec *codec)
 {
     TermWidget *term = currentTerminal();
-    if (term) {
+    if (term)
         term->setTextCodec(codec);
-    }
 }
-/*******************************************************************************
- 1. @函数:    setMismatchAlert
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-24
- 4. @说明:    是否将输入框标为警告
-*******************************************************************************/
+
 void TermWidgetPage::setMismatchAlert(bool alert)
 {
     m_findBar->setNoMatchAlert(alert);
 }
-/*******************************************************************************
- 1. @函数:    resizeEvent
- 2. @作者:    ut000439 王培利
- 3. @日期:    2020-02-24
- 4. @说明:    重绘时候，调整查找框的位置
-*******************************************************************************/
+
 void TermWidgetPage::resizeEvent(QResizeEvent *event)
 {
     Q_UNUSED(event)
-    //qDebug() << "resizeEvent" << x() << y();
-    this->m_findBar->move(width() - 382, 0);
+    this->m_findBar->move(width() - SEARCHBAR_RIGHT_MARGIN, 0);
 }
